#!/bin/bash

# Used by ipoffice service. Please keep in sync if you need to change it
# arg1 - process name
# arg2 - the file where the pid is saved
# Usage:  getpidbyname.sh -p processname -f pid_output_file


if [ -f /opt/Avaya/scripts/config_files/apps_logging_config ]; then
    . /opt/Avaya/scripts/config_files/apps_logging_config
fi

if [ -n ${ISA_LOG_PATH_NAME} -a -f ${ISA_LOG_PATH_NAME} ]; then
   logfile=${ISA_LOG_PATH_NAME}
else
   logfile=/tmp/.getprocessid
fi

function logevent () {
   local arg_l1="`date`"  #date
   local arg_l2=$1        #line n
   local arg_l3=$2        #event

   if [ -f "$logfile" ]; then
       echo "$$ || ${arg_l1} || $0 Line:${arg_l2} || ${arg_l3}"  >> $logfile
   fi
}

while getopts :p:f: option
do
    case "${option}" in
          p) processname=${OPTARG};;
          f) file=${OPTARG};;
          \?) echo "Invalid option: -$OPTARG" >&2
              logevent "$LINENO" "Invalid option: -$OPTARG"
              exit;;
    esac
done

# remove preview output file
if [ -f "$file" ]; then
    rm -rf "$file"
fi

exp='^[-~!@#$%^&*+]+[a-zA-Z]*$'

if [ -z "$processname" ]; then
    echo "Error: Empty process name is not allowed!"
    logevent "$LINENO" "Error: Empty process name is not allowed!"
    exit
fi

if [[ "$processname" =~ $exp ]]; then
    echo "Error: No valid process name format!"
    logevent "$LINENO" "Error: No valid process name format!"
    exit
fi

cmd=$(pgrep -x $processname)
if [ -z "$cmd" ]; then
  logevent "$LINENO" "the searched process was not found"
fi

if [ -n "$cmd" ]; then
    for pid in $cmd
    do
       pname=$(cat /proc/$pid/status | grep Name: | awk '{print $2}')
       logevent "$LINENO" "pname=$pname pid=$pid"
       if [ "$pname" == "$processname" ]; then
           if [ -z "$file" ]; then
               echo "$pid"
           else
               echo $pid > $file
           fi
           logevent "$LINENO" "matched $pname $pid"
       fi
    done
fi
