#!/usr/bin/perl

# Usage:    set_dhcp_ipaddr.pl <intf> [<peerdns>]
# Purpose:  Delete static IP address from network interface <intf> and configure it as DHCP client
# Works on:     CentOS 5.2, CentOS 4.6

use warnings;
use File::Copy;
use POSIX qw(:signal_h);

my($newHwAddr) = `LANGUAGE=en_US.UTF-8 ifconfig -a $ARGV[0] | grep -oE "HWaddr [0-9ABCDEF:]*" | grep -oE "[0-9ABCDEF:]*"`;
$filename1 = "/etc/sysconfig/network-scripts/ifcfg-$ARGV[0]";
$filename2 = "new-ifcfg-$ARGV[0]";
$filename3 = "$filename1.bkup";
$bootproto = "BOOTPROTO=dhcp\n";
$controlled = "NM_CONTROLLED=\"no\"\n";
$hwaddr = "HWADDR=$newHwAddr";

if ( ! $hwaddr =~ "\n" )
{
    $hwaddr = "$hwaddr \n";
}

if ( ! $ARGV[1] eq "") 
{
    $peerdns ="PEERDNS=$ARGV[1]\n";
}
else
{
    $gateway = "";
}


$cfgFileWasUpdated = 0;
unless (-e $filename1)
{
    $cfgFileWasUpdated = 1;
    if (open(file1, ">$filename1"))
    {
        ($ARGV[0] =~ '\\.') && print file1 "VLAN=yes\n";
        print file1 <<EOF;
DEVICE=$ARGV[0]
BOOTPROTO=dhcp
ONBOOT=yes
$hwaddr
EOF
        close file1;
  }
}

if (open(file1, $filename1) && open(file2, ">$filename2"))
{
    $cfgFileWasUpdated = 1;
    while($line = <file1>)
    {
        if ($line !~ /^ *((IPADDR)|(NETMASK)|(NETWORK)|(BROADCAST)|(PREFIXLEN)) *=/)
        {
            if ($line =~ /^ *BOOTPROTO *=/)
            {
                $line = $bootproto;
                $bootproto = '';
            }
            elsif ($line =~ /^ *NM_CONTROLLED *=/)
            {
                $line = $controlled;
                $controlled = '';
            }
            elsif ($line =~ /^ *HWADDR *=/)
            {
                $line = $hwaddr;
                $hwaddr = '';
            }
            elsif ($line =~ /^ *PEERDNS *=/)
            {
                $line = $peerdns;
                $peerdns = '';
            }
            if ($line ne "\n")
            {
                print file2 $line;
            }
        }
    }
    print file2 $bootproto;
    print file2 $controlled;
    print file2 $peerdns;
    close(file1);
    close(file2);
    unlink($filename3);
    move($filename2, $filename1);
}

if ($cfgFileWasUpdated != 0)
{
    $ss = POSIX::SigSet->new(SIGALRM);
    $dummy = POSIX::SigSet->new();
    sigprocmask(SIG_UNBLOCK, $ss, $dummy);
}

# Start DHCP client(s)
if (system("service network restart") != 0)
{
    print STDERR "service network restart has failed.\n";
}
