#!/bin/bash

##################################################
#
#  protocol://user_name:passwd@host:port/file_path local_dir
# 
# Usage: http <http_server> <user_name> <password> <remote_dir> <local_dir>"
# download ISO image file from remote directory on a server and
# prepare the file into a upgradable directory.
#
##################################################

g_NULL=/dev/null
declare -r cmd_DATE=/bin/date
declare -r cmd_FTP=/usr/bin/ftp
declare -r cmd_GREP=/bin/grep
declare -r cmd_RPM=/bin/rpm
declare -r cmd_CAT=/bin/cat
declare -r cmd_RM=/bin/rm
declare -r cmd_MKDIR=/bin/mkdir
declare -r cmd_WGET=/usr/bin/wget
declare -r cmd_AWK=/bin/awk
declare -r cmd_CUT=/bin/cut
declare -r cmd_SED=/bin/sed
declare -r cmd_MOUNT=/bin/mount
declare -r cmd_UMOUNT=/bin/umount
declare -r cmd_PERL=/usr/bin/perl
declare -r cmd_CP=/bin/cp
declare -r cmd_TOUCH=/bin/touch
declare -r cmd_UNZIP=/usr/bin/unzip
declare -r cmd_FIND=/bin/find
declare -r cmd_HEAD=/usr/bin/head
declare -r cmd_MKTEMP=/bin/mktemp
declare -r cmd_SCP=/usr/bin/scp
declare -r cmd_MD5SUM=/usr/bin/md5sum
declare -r cmd_PYTHON=/usr/bin/python
declare -r cmd_EJECT=/usr/sbin/eject
declare -r cmd_PING=/bin/ping
declare -r cmd_KILL=/bin/kill
declare -r cmd_CHECKISO=/usr/bin/checkisomd5
declare -r allow_DOWNLOAD=/opt/Avaya/.allowdownload

uwsfetch_LOGFILE=/opt/Avaya/uws_logs/uwsfetch.log

urldecode() {
    local data=${1//+/ }
    printf '%b' "${data//%/\x}"
}

urlencode()
{
echo $1 | $cmd_AWK '
    BEGIN {
	# We assume an awk implementation that is just plain dumb.
	# We will convert an character to its ASCII value with the
	# table ord[], and produce two-digit hexadecimal output
	# without the printf("%02X") feature.

	EOL = "%0A"		# "end of line" string (encoded)
	split ("1 2 3 4 5 6 7 8 9 A B C D E F", hextab, " ")
	hextab [0] = 0
	for ( i=1; i<=255; ++i ) ord [ sprintf ("%c", i) "" ] = i + 0
	if ("'"$EncodeEOL"'" == "yes") EncodeEOL = 1; else EncodeEOL = 0
    }
    {
	encoded = ""
	for ( i=1; i<=length ($0); ++i ) {
	    c = substr ($0, i, 1)
	    if ( c ~ /[a-zA-Z0-9.-]/ ) {
		encoded = encoded c		# safe character
	    } else if ( c == " " ) {
		encoded = encoded "+"	# special handling
	    } else {
		# unsafe character, encode it as a two-digit hex-number
		lo = ord [c] % 16
		hi = int (ord [c] / 16);
		encoded = encoded "%" hextab [hi] hextab [lo]
	    }
	}
	if ( EncodeEOL ) {
	    printf ("%s", encoded EOL)
	} else {
	    print encoded
	}
    }
    END {
    	#if ( EncodeEOL ) print ""
    }
'
#' "$@"
}

shlencode()
{
echo $1 | $cmd_AWK '
    BEGIN {
	# We assume an awk implementation that is just plain dumb.
	# We will convert an character to its ASCII value with the
	# table ord[], and produce two-digit hexadecimal output
	# without the printf("%02X") feature.

	EOL = "%0A"		# "end of line" string (encoded)
	split ("1 2 3 4 5 6 7 8 9 A B C D E F", hextab, " ")
	hextab [0] = 0
	for ( i=1; i<=255; ++i ) ord [ sprintf ("%c", i) "" ] = i + 0
	if ("'"$EncodeEOL"'" == "yes") EncodeEOL = 1; else EncodeEOL = 0
    }
    {
	encoded = ""
	for ( i=1; i<=length ($0); ++i ) {
	    c = substr ($0, i, 1)
	    if ( c ~ /[a-zA-Z0-9.-]/ ) {
		encoded = encoded c		# safe character
	    } else if ( c == " " ) {
		encoded = encoded "\\ "	# special handling
	    } else {
		# unsafe character, encode it as a two-digit hex-number
		#lo = ord [c] % 16
		#hi = int (ord [c] / 16);
		#encoded = encoded "%" hextab [hi] hextab [lo]
		encoded = encoded "\\" c
	    }
	}
	if ( EncodeEOL ) {
	    printf ("%s", encoded EOL)
	} else {
	    print encoded
	}
    }
    END {
    	#if ( EncodeEOL ) print ""
    }
'
#' "$@"
}

################################################################################
### Purpose: convert base64 encoded string 
### 
### Args:
###          $1 encoded string 
### Returns: decodded string

convert_sig_to_string()
{
  local INPUT_STR=`echo -n $1 | /usr/bin/base64 -d`
  if [ ! "$?" = "0" ] ; then
    echo -n "FAIL"
  fi
  echo -n $INPUT_STR
}

################################################################################
### Purpose: add a log entry into the file /opt/Avaya/uws_logs/uwsfetch.log
### 
### Args:
###          $1 log string 
### Returns: 

addlog()
{
  local timestamp=$( date '+%Y-%m-%dT%H:%M:%S' )
  local logstr=$1
  echo "<log>$timestamp $logstr</log>" | ${cmd_SED} 's/$/&\n/' >> $uwsfetch_LOGFILE
}

adderrlog()
{
  local timestamp=$( date '+%Y-%m-%dT%H:%M:%S' )
  local logcode=$1
  case "$logcode" in
  1) logstr="Failed due to incorrect parameters"
     ;;
  2) logstr="Failed due to wrong protocol"
     ;;
  3) logstr="Failed due to no ISO image file name specified"
     ;;
  4) logstr="Failed due to no connectivity to host"
     ;;
  5) logstr="Failed to fetch ISO file from ftp server"
     ;;
  6) logstr="Failed to create repo data"
     ;;
  7) logstr="Failed to fetch ISO file from http server"
     ;;
  8) logstr="Failed to fetch ISO file from sftp server"
     ;;
  9) logstr="Failed to fetch ISO file from DVD drive"
     ;;
  10) logstr="Failed to fetch ISO file from usb drive"
     ;;
  11) logstr="Failed to create REPO due to current version is higher"
     ;;
  12) logstr="Failed to connect server due to wrong port number"
     ;;
  13) logstr="Operation is cancelled"
     ;;
  14) logstr="Failed to validate ISO MD5 checksum"
     ;;
  *) logstr="Failed due to unspecified reason"
     ;;
  esac
  echo "<error><code>$logcode</code><desc>$logstr</desc></error>" | ${cmd_SED} 's/$/&\n/' >> $uwsfetch_LOGFILE
}

setlog()
{
  if [ -f "${uwsfetch_LOGFILE}" ]; then
      ${cmd_RM} -f ${uwsfetch_LOGFILE}
      ${cmd_TOUCH} ${uwsfetch_LOGFILE}
  fi
}


################################################################################
### Purpose: extract substring between prefix and suffix.
### 
### Args:
###          $1 string to extract from 
###          $2 prefix
###          $3 suffix
###
### Returns: substring

fn_parse_URI()
{
    local -r URI=$1
    local -r _arg_passback_protocol=$2
    local -r _arg_passback_username=$3
    local -r _arg_passback_password=$4
    local -r _arg_passback_host=$5
    local -r _arg_passback_remotepath=$6
    local -r _arg_passback_port=$7

    # parse URI to get protocol
    eval "${_arg_passback_protocol}=${URI%"://"*}"

    # parse URI to get remote path
    prefix=${URI%"/"*}
    eval "${_arg_passback_remotepath}=${URI#${prefix}"/"}"

    # parse URI to get user info if existing
    echo $URI | $cmd_GREP "@" > $g_NULL
    if [ $? -eq 0 ]; then
      # if URI contains @, then parse username and password
      noprefix=${URI#${URI%"://"*}"://"}
      user_info=${noprefix%"@"*}
      eval "${_arg_passback_username}=${user_info%":"*}"
      eval "${_arg_passback_password}=${user_info#*":"}"

      # get host
      noprefix=${URI#${URI%"@"*}"@"}
      host_info=${noprefix%%/*}
      echo $host_info | $cmd_GREP ":" > $g_NULL
      if [ $? -eq 0 ]; then
        eval "${_arg_passback_host}=${host_info%":"*}"
        eval "${_arg_passback_port}=${host_info#*":"}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      else
        eval "${_arg_passback_host}=${host_info#*"@"}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      fi
    else
      noprefix=${URI#${URI%"://"*}"://"}
      user_info=""
      eval "${_arg_passback_username}=${user_info%":"*}"
      eval "${_arg_passback_password}=${user_info#*":"}"
      host_info=${noprefix%%/*}
      echo $host_info | $cmd_GREP ":" > $g_NULL
      if [ $? -eq 0 ]; then
        eval "${_arg_passback_host}=${host_info%":"*}"
        eval "${_arg_passback_port}=${host_info#*":"}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      else
        eval "${_arg_passback_host}=${noprefix%%/*}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      fi
    fi
    return 0
}

################################################################################
### Purpose: find the first USB device that contains an iso image file.
### Args:
###          $1 device_name
###
### Returns: device name

fn_find_usb_device()
{
    local -r FILENM=$1
    DEVICENM=NULL
    for udi in $(/usr/bin/hal-find-by-capability --capability storage)
    do
        device=$(hal-get-property --udi $udi --key block.device)
        vendor=$(hal-get-property --udi $udi --key storage.vendor)
        model=$(hal-get-property --udi $udi --key storage.model)
        if [[ $(hal-get-property --udi $udi --key storage.bus) = "usb" ]]
        then
            parent_udi=$(hal-find-by-property --key block.storage_device --string $udi)
            mount=$(hal-get-property --udi $parent_udi --key volume.mount_point)
            label=$(hal-get-property --udi $parent_udi --key volume.label)
            media_size=$(hal-get-property --udi $udi --key storage.removable.media_size)
            lmpoint=`$cmd_MKTEMP -d`
            udevname=${device}1
            $cmd_MOUNT $udevname $lmpoint
            if [[ $? -ne 0 ]]; then
               udevname=${device}2
               $cmd_MOUNT $udevname $lmpoint
            fi
            if [[ $? -eq 0 ]]; then
                if [[ -z $FILENM || $FILENM == "/" ]]; then
                   DEVICENM=$udevname
                   $cmd_UMOUNT $lmpoint
                   if [ $? -ne 0 ]; then
                      addlog "Warning umount error ${lmpoint}"
                   fi
                   $cmd_RM -rf $lmpoint
                   addlog "Found USB: $vendor $model $device"
                   break
                else
                   rtn=`$cmd_FIND $lmpoint -name $FILENM -exec echo found \\;`
                   if [[ $rtn == "found" ]]; then
                       DEVICENM=$udevname
                       $cmd_UMOUNT $lmpoint
                       if [ $? -ne 0 ]; then
                          addlog "Warning umount error ${lmpoint}"
                       fi
                       $cmd_RM -rf $lmpoint
                       addlog "Found USB: $vendor $model $device"
                       break
                   fi
                fi
            fi
            $cmd_UMOUNT $lmpoint
            if [ $? -ne 0 ]; then
               addlog "Warning umount error ${lmpoint}"
            fi
            $cmd_RM -rf $lmpoint
        fi
    done
    echo $DEVICENM
}


################################################################################
### Purpose: validate MD5 checksum 
### Args:
###          $1 file to be validated
###          $2 MD5 checksum file
### Returns: substring

fn_validate_checksum()
{
    local -r FILEDATA=$1
    local -r FILEMD5S=$2
    if [[ ! -e "$FILEDATA" ]]; then
       return 1;
    fi
    if [[ ! -e "$FILEMD5S" ]]; then
       return 2;
    fi
    newmd5sum=`${cmd_MD5SUM} ${FILEDATA} | ${cmd_AWK} '{print $1}'`
    oldmd5sum=`${cmd_CAT} ${FILEMD5S} | ${cmd_HEAD} -n 1 | ${cmd_AWK} '{print $1}'`
    if [ "$newmd5sum" != "$oldmd5sum" ]; then
       return 3;
    fi
    return 0;
}

################################################################################
### Purpose: download the file 
###          and save in local directory
### Args:
###          $1 protocol
###          $2 host
###          $3 user_name
###          $4 password
###          $5 remote_path
###          $6 file_path
###          $7 file_name
###          $8 local_dir
###          $9 proxyurl
###
### Returns: substring

fn_http_download()
{
    local -r PROTOCOL=$1
    local -r HOST=$2
    local -r USERNM=$3
    local -r PASSWD=$4
    local -r REMOTEDIR=$5
    local -r FILEPH=$6
    local -r FILENM=$7
    local -r LOCALDIR=$8
    local -r PROXYSET=$9
    
    # assume password is urlencoded.
    #EUSERNM=`urlencode ${USERNM}`
    #EPASSWD=`urlencode ${PASSWD}`
    EUSERNM=${USERNM}
    EPASSWD=${PASSWD}
    
    #MD5FILENM=${FILENM%.*}.md5

    if [ $USERNM == "anonymous" ]; then
       if [ $FILEPH == "/" ]; then
          URI=$PROTOCOL://${HOST}${FILEPH}${FILENM}
          #MD5URI=$PROTOCOL://${HOST}${FILEPH}${MD5FILENM}
       else
          URI=$PROTOCOL://${HOST}${FILEPH}/${FILENM}
          #MD5URI=$PROTOCOL://${HOST}${FILEPH}/${MD5FILENM}
       fi
    else
       if [ $FILEPH == "/" ]; then
          URI=$PROTOCOL://$EUSERNM:$EPASSWD@${HOST}${FILEPH}${FILENM}
          #MD5URI=$PROTOCOL://$EUSERNM:$EPASSWD@${HOST}${FILEPH}${MD5FILENM}
       else
          URI=$PROTOCOL://$EUSERNM:$EPASSWD@${HOST}${FILEPH}/${FILENM}
          #MD5URI=$PROTOCOL://$EUSERNM:$EPASSWD@${HOST}${FILEPH}/${MD5FILENM}
       fi
    fi
    # set proxy
    if [ -n "$PROXYSET" ]; then
       export http_proxy="$PROXYSET"
       addlog "setting $exportcmd"
    else
       unset http_proxy
    fi
    # get file with wget
    addlog "Retrieving the ${FILENM} from ${HOST}${FILEPH}/${FILENM}"
    $cmd_WGET --no-check-certificate $URI -O ${LOCALDIR}/${FILENM}
    if [ $? -ne 0 ]; then
       addlog "Failed to retrieve from http server: ${HOST}"
       return 11
    fi
    #addlog "Retrieving the ${MD5FILENM} from ${HOST}"
    #$cmd_WGET $MD5URI -O ${LOCALDIR}/${MD5FILENM}
    #if [ $? -ne 0 ]; then
    #   addlog "Warning retrieving MD5 file error from http server: ${HOST}"
    #else
    #   fn_validate_checksum ${LOCALDIR}/${FILENM} ${LOCALDIR}/${MD5FILENM}
    #   if [ $? -ne 0 ]; then
    #      addlog "Failed to validate MD5 checksum: ${FILENM}"
    #      return 12
    #   fi
    #fi
    ${cmd_CHECKISO} ${LOCALDIR}/${FILENM}
    if [ $? -ne 0 ]; then
       addlog "Failed to validate ISO MD5 checksum: ${FILENM}"
       return 12
    fi
    return 0
}

################################################################################
### Purpose: ftp download the file using wget
###          and save in local directory
### Args:
###          $1 protocol
###          $2 host
###          $3 user_name
###          $4 password
###          $5 remote_path
###          $6 file_path
###          $7 file_name
###          $8 local_dir
###          $9 proxyurl
###
### Returns: substring

fn_newftp_download()
{
    local -r PROTOCOL=$1
    local -r HOST=$2
    local -r USERNM=$3
    local -r PASSWD=$4
    local -r REMOTEDIR=$5
    local -r FILEPH=$6
    local -r FILENM=$7
    local -r LOCALDIR=$8
    local -r PROXYSET=$9
    
    # assume password is urlencoded.
    #EUSERNM=`urlencode ${USERNM}`
    #EPASSWD=`urlencode ${PASSWD}`
    EUSERNM=${USERNM}
    DPASSWD="$(urldecode $PASSWD)"
    
    #MD5FILENM=${FILENM%.*}.md5

    if [ $FILEPH == "/" ]; then
       URI=$PROTOCOL://${HOST}${FILEPH}${FILENM}
       #MD5URI=$PROTOCOL://${HOST}${FILEPH}${MD5FILENM}
    else
       URI=$PROTOCOL://${HOST}${FILEPH}/${FILENM}
       #MD5URI=$PROTOCOL://${HOST}${FILEPH}/${MD5FILENM}
    fi
    # set proxy
    if [ -n "$PROXYSET" ]; then
       export http_proxy="$PROXYSET"
       addlog "setting $exportcmd"
    else
       unset http_proxy
    fi
    # get file with wget
    addlog "Retrieving the ${FILENM} from ${HOST}${FILEPH}${FILENM}"
    $cmd_WGET -t 2 --ftp-user=${USERNM} --ftp-password=${DPASSWD} $URI -O ${LOCALDIR}/${FILENM}
    if [ $? -ne 0 ]; then
       addlog "Failed to retrieve from ftp server: ${HOST}"
       return 71
    fi
    #addlog "Retrieving the ${MD5FILENM} from ${HOST}"
    #$cmd_WGET -t 2 --ftp-user=${USERNM} --ftp-password=${DPASSWD} $MD5URI -O ${LOCALDIR}/${MD5FILENM}
    #if [ $? -ne 0 ]; then
    #   addlog "Warning retrieving MD5 file error from ftp server: ${HOST}"
    #else
    #   fn_validate_checksum ${LOCALDIR}/${FILENM} ${LOCALDIR}/${MD5FILENM}
    #   if [ $? -ne 0 ]; then
    #      addlog "Failed to validate MD5 checksum: ${FILENM}"
    #      return 12
    #   fi
    #fi
    ${cmd_CHECKISO} ${LOCALDIR}/${FILENM}
    if [ $? -ne 0 ]; then
       addlog "Failed to validate ISO MD5 checksum: ${FILENM}"
       return 72
    fi
    return 0
}

################################################################################
### Purpose: download the file from ftp server
###          and save in current local directory
### Args:
###          $1 host
###          $2 user_name
###          $3 password
###          $4 remote_path
###          $5 file_name
###          $6 localdir
###
### Returns: substring

fn_ftp_download()
{
    local -r HOST=$1
    local -r USERNM=$2
    local -r PASSWD=$3
    local -r REMOTEDIR=$4
    local -r FILENM=$5
    local -r LDIRNM=$6

    DPASSWD="$(urldecode $PASSWD)"
    MD5FILENM=${FILENM%.*}.md5

    # get the file 
    addlog "Copying ${file_name} from ftp server ${HOST} ${remote_path}"
    list_file=$(echo $RANDOM)
$cmd_FTP -inv $HOST <<END_SCRIPT > $list_file
        quote USER $USERNM
        quote PASS $DPASSWD
        lcd $LDIRNM
        cd $REMOTEDIR
        bin
        get $FILENM
        get $MD5FILENM
        bye
END_SCRIPT

    # check error
    errstring="Host name lookup failure"
    $cmd_CAT $list_file | $cmd_GREP "${errstring}" > $g_NULL
    if [ $? -eq 0 ]; then
       addlog "Failed to connect to ftp server: ${HOST}"
       $cmd_RM -f $list_file
       return 21
    fi
    errstring="No route to host"
    $cmd_CAT $list_file | $cmd_GREP "${errstring}" > $g_NULL
    if [ $? -eq 0 ]; then
       addlog "Failed to connect to ftp server: ${HOST}"
       $cmd_RM -f $list_file
       return 21
    fi
    errstring="Connection timed out"
    $cmd_CAT $list_file | $cmd_GREP "${errstring}" > $g_NULL
    if [ $? -eq 0 ]; then
       addlog "Failed to connect to ftp server: ${HOST}"
       $cmd_RM -f $list_file
       return 21
    fi
    errstring="Failed to change"
    $cmd_CAT $list_file | $cmd_GREP "${errstring}" > $g_NULL
    if [ $? -eq 0 ]; then
       addlog "Failed to change remote dir: ${REMOTEDIR}/${FILENM}"
       $cmd_RM -f $list_file
       return 21
    fi
    errstring="Failed to"
    $cmd_CAT $list_file | $cmd_GREP "${errstring}" > $g_NULL
    if [ $? -eq 0 ]; then
       addlog "Failed to ftp file: ${REMOTEDIR}/${FILENM}"
       $cmd_RM -f $list_file
       return 21
    fi
    
    $cmd_RM -f $list_file

    if [ -e ${REMOTEDIR}/${MD5FILENM} ]; then
       fn_validate_checksum ${LOCALDIR}/${FILENM} ${LOCALDIR}/${MD5FILENM}
       if [ $? -ne 0 ]; then
          addlog "Failed to validate MD5 checksum: ${FILENM}"
          return 22
       fi
    else
       addlog "Warning ftp MD5 file error from http server: ${HOST}${LOCALDIR}/${MD5FILENM}"
    fi
    return 0
}

################################################################################
### Purpose: download file based on types from usb drive
###          and save in current local directory
### Args:
###          $1 remote_path
###          $2 local_dir
###          $3 file_name
###
### Returns: substring

fn_usb_download()
{
    local -r remote_path=$1
    local -r LOCALDIR=$2
    local -r _arg_passback_filename=$3

    # mount the usb drive
    local localdv=/dev/sdb
    localmp=`$cmd_MKTEMP -d`
    # find a device
    lufile_name=$(basename $remote_path)
    lufile_xtsn=${lufile_name##*.}
    #if [[ $lufile_xtsn != "iso" && $lufile_xtsn != "ISO" ]]; then
    #   lufile_name="*.iso"
    #fi
    localdv=$(fn_find_usb_device $lufile_name)
    if [ $localdv == "NULL" ]; then
       $cmd_RM -rf $localmp
       addlog "Failed to find an USB drive that contains $file_name"
       return 31
    fi
    lmcmd="${cmd_MOUNT} ${localdv} ${localmp}"
    #addlog "${lmcmd}"
    ${lmcmd}
    if [ $? -ne 0 ]; then
       $cmd_RM -rf $localmp
       addlog "Failed to mount the USB: ${lmcmd}"
       return 31
    fi
    if [[ $lufile_xtsn == "iso" || $lufile_xtsn == "ISO" ]]; then
       # copy file
       addlog "Copying ${lufile_name} from USB drive ${remote_path}"
       ${cmd_CP} -f ${localmp}/${remote_path} ${LOCALDIR}/${lufile_name}
       rc=$?
       # unmount the drive
       ${cmd_UMOUNT} ${localmp}
       if [ $? -ne 0 ]; then
          addlog "Warning umount error ${localmp}"
       fi
       $cmd_RM -rf $localmp
       if [ $rc -ne 0 ]; then
          addlog "Failed to copy file from USB dir: ${remote_path}"
          return $rc
       fi
       eval "${_arg_passback_filename}=$lufile_name"
    else
       lufile_name=`$cmd_FIND ${localmp} -name "*.iso"`
       if [[ -z "$lufile_name" ||
             $lufile_name == "${localmp}/images/boot.iso" ]]; then
          # unmount the drive
          ${cmd_UMOUNT} ${localmp}
          if [ $? -ne 0 ]; then
             addlog "Warning umount error ${localmp}"
          fi
          $cmd_RM -rf $localmp
          addlog "Cannot find ISO image and use expanded USB drive: ${localdv}"
          eval "${_arg_passback_filename}=$localdv"
          return 1
       fi
       if [ -f ${localmp}/avaya/ABE_release ]; then
               exp_ver_file="${localmp}/avaya/ABE_release"
       elif [ -f ${localmp}/avaya/APC_release ]; then
               exp_ver_file="${localmp}/avaya/APC_release"
       elif [ -f ${localmp}/avaya/Avaya_release ]; then
               exp_ver_file="${localmp}/avaya/Avaya_release"
       fi
       if [[ ! -e $exp_ver_file ]]; then
          exp_ver_bdver="0"
          exp_ver_major="0"
          exp_ver_major="0"
       else
          if [ -f ${localmp}/avaya/ABE_release ]; then
               exp_ver_ver=`grep "ABE_VERSION=" ${localmp}/avaya/ABE_release | awk '{ print $1}' | cut -d= -f2 | cut -d')' -f1`
          elif [ -f ${localmp}/avaya/APC_release ]; then
               exp_ver_ver=`grep "APC_VERSION=" ${localmp}/avaya/APC_release | awk '{ print $1}' | cut -d= -f2 | cut -d')' -f1`
          elif [ -f ${localmp}/avaya/Avaya_release ]; then
               exp_ver_ver=`grep "ABE_VERSION=" ${localmp}/avaya/Avaya_release | awk '{ print $1}' | cut -d= -f2 | cut -d')' -f1`
          fi
          exp_ver_mmver=`echo $exp_ver_ver | cut -d'\\' -f1`
          exp_ver_bdver=`echo $exp_ver_ver | cut -d'\\' -f2`
          exp_ver_bdver=`echo $exp_ver_bdver | cut -d'(' -f2`
          exp_ver_bdver=`echo $exp_ver_bdver | cut -d'_' -f1`
          exp_ver_major="${exp_ver_mmver%%.*}"
          exp_ver_nrest="${exp_ver_mmver#*.}"
          exp_ver_minor="${exp_ver_nrest%%.*}"
       fi
       usbtarget=""
       for lufile_one in $lufile_name
       do
          lufile_base=$(basename "$lufile_one")
          isonamepttn='abe-[0-9]+\.[0-9]\.[0-9]-[0-9]+_el6.iso'
          if [[ $lufile_base =~ $isonamepttn ]]; then
             iso_ver_mmver=`echo $lufile_base | cut -d'-' -f2`
             iso_ver_major="${iso_ver_mmver%%.*}"
             iso_ver_nrest="${iso_ver_mmver#*.}"
             iso_ver_minor="${iso_ver_nrest%%.*}"
             iso_ver_mmver=`echo $lufile_base | cut -d'-' -f3`
             iso_ver_bdver="${iso_ver_mmver%%_*}"
             if [[ $iso_ver_major -gt $exp_ver_major ]]; then
                usbtarget="$lufile_one"
                exp_ver_major=$iso_ver_major
                exp_ver_minor=$iso_ver_minor
                exp_ver_bdver=$iso_ver_bdver
             else
                if [[ $iso_ver_major -eq $exp_ver_major && $iso_ver_minor -gt $exp_ver_minor ]]; then
                   usbtarget="$lufile_one"
                   exp_ver_major=$iso_ver_major
                   exp_ver_minor=$iso_ver_minor
                   exp_ver_bdver=$iso_ver_bdver
                else
                   if [[ $iso_ver_major -eq $exp_ver_major && $iso_ver_minor -eq $exp_ver_minor && $iso_ver_bdver -gt $exp_ver_bdver ]]; then
                      usbtarget="$lufile_one"
                      exp_ver_major=$iso_ver_major
                      exp_ver_minor=$iso_ver_minor
                      exp_ver_bdver=$iso_ver_bdver
                   fi
                fi
             fi
          fi
       done
       if [ -z "$usbtarget" ]; then
          # unmount the drive
          ${cmd_UMOUNT} ${localmp}
          if [ $? -ne 0 ]; then
             addlog "Warning umount error ${localmp}"
          fi
          $cmd_RM -rf $localmp
          #addlog "Failed to find ISO image file from USB dir: ${remote_path}"
          #return 32
          addlog "Cannot find ISO image and use expanded USB drive: ${localdv}"
          eval "${_arg_passback_filename}=$localdv"
          return 1
       fi
       # copy file
       addlog "Copying ${usbtarget} from USB drive ${remote_path}"
       ${cmd_CP} -f ${usbtarget} ${LOCALDIR}/.
       rc=$?
       # unmount the drive
       ${cmd_UMOUNT} ${localmp}
       if [ $? -ne 0 ]; then
          addlog "Warning umount error ${localmp}"
       fi
       $cmd_RM -rf $localmp
       if [ $rc -ne 0 ]; then
          addlog "Failed to copy file from USB dir: ${remote_path}"
          return $rc
       fi
       eval "${_arg_passback_filename}=$(basename ${usbtarget})"
    fi
    return 0
}

################################################################################
### Purpose: download file based on types from DVD drive
###          and save in current local directory
### Args:
###          $1 remote_path
###          $2 local_dir
###          $3 file_name
###
### Returns: substring

fn_dvd_download()
{
    local -r remote_path=$1
    local -r LOCALDIR=$2
    local -r _arg_passback_filename=$3

    # mount the dvd drive
    local -r localdv=/dev/dvd
    localmp=`$cmd_MKTEMP -d`
    lmcmd="${cmd_MOUNT} ${localdv} ${localmp}"
    ${lmcmd}
    if [ $? -ne 0 ]; then
       $cmd_RM -rf $localmp
       addlog "Failed to mount the DVD: ${lmcmd}"
       return 31
    fi
    ldfile_name=$(basename $remote_path)
    ldfile_xtsn=${ldfile_name##*.}
    if [[ $ldfile_xtsn == "iso" || $ldfile_xtsn == "ISO" ]]; then
       # copy file
       addlog "Copying ${ldfile_name} from DVD drive ${remote_path}"
       ${cmd_CP} -f ${localmp}/${remote_path} ${LOCALDIR}/${ldfile_name}
       rc=$?
       # unmount the drive
       ${cmd_UMOUNT} ${localmp}
       if [ $? -ne 0 ]; then
          addlog "Warning umount error ${localmp}"
       fi
       $cmd_RM -rf $localmp
       if [ $rc -ne 0 ]; then
          addlog "Failed to copy file from DVD dir: ${remote_path}"
          return $rc
       fi
       eval "${_arg_passback_filename}=$ldfile_name"
    else
       ldfile_name=`$cmd_FIND ${localmp} -name "*.iso"`
       if [[ -z "$ldfile_name" || 
             $(basename $ldfile_name) == "boot.iso" ]]; then
          # unmount the drive
          ${cmd_UMOUNT} ${localmp}
          if [ $? -ne 0 ]; then
             addlog "Warning umount error ${localmp}"
          fi
          $cmd_RM -rf $localmp
          addlog "Cannot find ISO image and use expanded DVD dir: ${localdv}"
          eval "${_arg_passback_filename}=$localdv"
          return 1
       fi
       ldfile_el6n=`echo $ldfile_name | $cmd_GREP el6`
       if [ -z "$ldfile_el6n" ]; then
          ldfile_el6n=`echo $ldfile_name | $cmd_HEAD -n 1`
       fi
       if [ -z "$ldfile_el6n" ]; then
          # unmount the drive
          ${cmd_UMOUNT} ${localmp}
          if [ $? -ne 0 ]; then
             addlog "Warning umount error ${localmp}"
          fi
          $cmd_RM -rf $localmp
          addlog "Failed to find ISO image and expand from DVD: ${localdv}"
          eval "${_arg_passback_filename}=$localdv"
          return 1
       fi
       # copy file
       addlog "Copying ${ldfile_el6n} from DVD drive ${remote_path}"
       ${cmd_CP} -f ${ldfile_el6n} ${LOCALDIR}/.
       rc=$?
       # unmount the drive
       ${cmd_UMOUNT} ${localmp}
       if [ $? -ne 0 ]; then
          addlog "Warning umount error ${localmp}"
       fi
       $cmd_RM -rf $localmp
       if [ $rc -ne 0 ]; then
          addlog "Failed to copy file from DVD dir: ${ldfile_el6n}"
          return $rc
       fi
       eval "${_arg_passback_filename}=$(basename ${ldfile_el6n})"
    fi
    return 0
}

################################################################################
### Purpose: download file from scp server
###          and save in current local directory
### Args:
###          $1 host
###          $2 user_name
###          $3 password
###          $4 remote_path
###          $5 local_dir
###          $6 file_name
###          $7 port
###
### Returns: substring

fn_scp_download()
{
    local -r HOST=$1
    local -r USERNM=$2
    local -r PASSWD=$3
    local -r REMOTEDIR=$4
    local -r LOCALDIR=$5
    local -r FILENM=$6
    local -r PORTN=$7

    local -r PSSHPASS=/usr/lib/ipoffice/psshpass.py

    DPASSWD="$(urldecode $PASSWD)"

    OPORT=$PORTN
    if [[ -z "$PORTN" ]]; then
       OPORT=22
    fi

    # command
    addlog "Copying ${FILENM} from ${HOST}:${REMOTEDIR}"
    ${cmd_PYTHON} ${PSSHPASS} "${DPASSWD}" -o "StrictHostKeyChecking no" -o "NumberOfPasswordPrompts=1" -o "ConnectTimeout=5" -v -P ${OPORT} ${USERNM}@${HOST}:${REMOTEDIR} ${LOCALDIR}/${FILENM}
    if [ $? -ne 0 ]; then
       addlog "Failed to scp the file: ${precmd}"
       return 61
    fi

    #MD5FILENM=${FILENM%.*}.md5
    #MD5FILEPH=${REMOTEDIR%/*}
    #addlog "Copying ${MD5FILENM} from ${HOST}:${MD5FILEPH}/${MD5FILENM}"
    #${cmd_PYTHON} ${PSSHPASS} "${DPASSWD}" -o "StrictHostKeyChecking no" -v ${USERNM}@${HOST}:${MD5FILEPH}/${MD5FILENM} ${LOCALDIR}/${MD5FILENM}
    #if [ $? -ne 0 ]; then
    #   addlog "Warning scp MD5 file error: ${USERNM}@${HOST}:${MD5FILEPH}/${MD5FILENM}"
    #else
    #   fn_validate_checksum ${LOCALDIR}/${FILENM} ${LOCALDIR}/${MD5FILENM}
    #   if [ $? -ne 0 ]; then
    #      addlog "Failed to validate MD5 checksum: ${FILENM}"
    #      return 62
    #   fi
    #fi
    ${cmd_CHECKISO} ${LOCALDIR}/${FILENM}
    if [ $? -ne 0 ]; then
       addlog "Failed to validate ISO MD5 checksum: ${FILENM}"
       return 62
    fi

    return 0
}

################################################################################
### Purpose: download file from sftp server
###          and save in current local directory
### Args:
###          $1 host
###          $2 user_name
###          $3 password
###          $4 remote_path
###          $5 local_dir
###          $6 file_name
###          $7 port
###
### Returns: substring

fn_sftp_download()
{
    local -r HOST=$1
    local -r USERNM=$2
    local -r PASSWD=$3
    local -r REMOTEDIR=$4
    local -r LOCALDIR=$5
    local -r FILENM=$6
    local -r PORTN=$7

    local -r PSSHPASS=/usr/lib/ipoffice/psshsftp.py

    DPASSWD="$(urldecode $PASSWD)"

    OPORT=$PORTN
    if [[ -z "$PORTN" ]]; then
       OPORT=22
    fi

    # command
    addlog "Copying ${FILENM} from ${HOST}:${REMOTEDIR}"
    ${cmd_PYTHON} ${PSSHPASS} ${HOST} ${OPORT} "${USERNM}" "${DPASSWD}" get "${REMOTEDIR}" "${LOCALDIR}/${FILENM}"
    if [ $? -ne 0 ]; then
       addlog "Failed to sftp the file: ${REMOTEDIR} from $HOST"
       return 61
    fi

    ${cmd_CHECKISO} ${LOCALDIR}/${FILENM}
    if [ $? -ne 0 ]; then
       addlog "Failed to validate ISO MD5 checksum: ${FILENM}"
       return 62
    fi

    return 0
}

################################################################################
### Purpose: apply the donwloaded files
### Args:
###          $1 file name
###          $2 localdir
###          $3 expanded
###
### Returns: substring

fn_apply()
{
    FILENM=$1
    LCLDIR=$2
    EXPAND=$3
    apath=/usr/lib/ipoffice/createUwsArchive.pl
    sfile=/usr/lib/ipoffice/UpPackInv.xsd
    prepo=/var/www/html/avaya/uws
    mpont=`$cmd_MKTEMP -d`
    if [ -z $EXPAND ]; then
       EXPAND=0;
    fi

    if [ $EXPAND -ne 1 ]; then
       mountcmd="${cmd_MOUNT} -o loop ${LCLDIR}/${FILENM} ${mpont}"
       ${mountcmd}
    else
       mountcmd="${cmd_MOUNT} ${FILENM} ${mpont}"
       ${mountcmd}
    fi
    if [ $? -ne 0 ]; then
       addlog "Failed to mount the file: ${mountcmd}"
       $cmd_RM -rf $mpont
       return 51
    fi

    rc=0
    #relnum=$(echo ${FILENM} | sed -e 's/^...[^0-9]//' | sed -e 's/_..*$//')
    #cleancmd="${cmd_RM} -rf ${prepo}/${relnum}"
    #${cleancmd}
    #crepocmd="${cmd_PERL} ${apath} ${sfile} ${mpont} ${prepo} 2 abe"
    #${crepocmd}
    #if [ $? -ne 0 ]; then
    #   addlog "Failed to create the repo: ${crepocmd}"
    #   rc=52
    #fi

    addlog "Creating the repo data"
    APPS_DIR=${mpont}/avaya
    if [ -f $APPS_DIR/ABE_release -o -f $APPS_DIR/APC_release -o -f $APPS_DIR/Avaya_release ]; then
          # release properties on source media 
          if [ -f $APPS_DIR/ABE_release ]; then
               MRVERSION=ABE_VERSION
               MRFILE=ABE_release
               MRTYPE=abe
          elif [ -f $APPS_DIR/APC_release ]; then
               MRVERSION=APC_VERSION
               MRFILE=APC_release
               MRTYPE=apc
          elif [ -f $APPS_DIR/Avaya_release ]; then
               # TODO check against ignited server type
               MRFILE=Avaya_release
          fi
          UWS_REPO="$prepo"
          UWS_VERSION=`grep "$MRVERSION=" $APPS_DIR/$MRFILE | awk '{ print $1}' | cut -d'=' -f2 | cut -d')' -f1`
          UWS_MMVER=`echo $UWS_VERSION | cut -d'\\' -f1`
          UWS_BDVER=`echo $UWS_VERSION | cut -d'\\' -f2`
          UWS_BDVER=`echo $UWS_BDVER | cut -d'(' -f2`
          UWS_BDVER=`echo $UWS_BDVER | cut -d'_' -f1`
          # check against current version
          if [ -f /opt/Avaya/APC_release ]; then
               CVFILE=/opt/Avaya/APC_release
               CRVERSION=APC_VERSION
               CRTYPE=apc
          elif [ -f /opt/Avaya/ABE_release -o -f /opt/Avaya/APPL_release ]; then
               CVFILE=/opt/Avaya/ABE_release
               CRVERSION=ABE_VERSION
               CRTYPE=abe
          elif [ -f /opt/Avaya/Avaya_release ]; then
               # TODO check against ignited server type
               CVFILE=/opt/Avaya/Avaya_release
          fi
          if [ -n "$MRTYPE" -a -n "$CRTYPE" -a "$MRTYPE" != "$CRTYPE" ]; then
               addlog "Failed to create file repository for current ignited server type."
               rc=14
          fi
          if [ -e $CVFILE -a -n "$CRVERSION" ]; then
             CVC_VERSION=`grep "$CRVERSION=" $CVFILE | awk '{ print $1}' | cut -d= -f2 | cut -d')' -f1`
             CVC_MMVER=`echo $CVC_VERSION | cut -d'\\' -f1`
             CVC_BDVER=`echo $CVC_VERSION | cut -d'\\' -f2`
             CVC_BDVER=`echo $CVC_BDVER | cut -d'(' -f2`
             CVC_BDVER=$((10#`echo $CVC_BDVER | cut -d'_' -f1`))
             CVC_MAJOR=$((10#`echo "${CVC_MMVER%%.*}"`))
             CVC_NREST="${CVC_MMVER#*.}"
             CVC_MINOR=$((10#`echo "${CVC_NREST%%.*}"`))
             CVC_MNT="${CVC_NREST##*.}"
             CVC_MNT="$(( $((10#$CVC_MNT)) / 100 ))"
             CVC_SPEC="${CVC_NREST##*.}"
             CVC_SPEC="$(( $((10#$CVC_SPEC)) % 100 ))"
             UWS_MAJOR=$((10#`echo "${UWS_MMVER%%.*}"`))
             UWS_NREST="${UWS_MMVER#*.}"
             UWS_MINOR=$((10#`echo "${UWS_NREST%%.*}"`))
             UWS_MNT="${UWS_NREST##*.}"
             UWS_MNT="$(( $((10#$UWS_MNT)) / 100 ))"
             UWS_SPEC="${UWS_NREST##*.}"
             UWS_SPEC="$(( $((10#$UWS_SPEC)) % 100 ))"
             # echo "CVC_MAJOR=${CVC_MAJOR}; CVC_MINOR=${CVC_MINOR}; CVC_MNT=${CVC_MNT}; CVC_SPEC=${CVC_SPEC}; CVC_BDVER=${CVC_BDVER};"
             # echo "UWS_MAJOR=${UWS_MAJOR}; UWS_MINOR=${UWS_MINOR}; UWS_MNT=${UWS_MNT}; UWS_SPEC=${UWS_SPEC}; UWS_BDVER=${UWS_BDVER};"

            CVC_NUM=$((${CVC_MAJOR}*10000000000 + ${CVC_MINOR}*100000000 + ${CVC_MNT}*1000000 + ${CVC_SPEC}*10000 + ${CVC_BDVER}))
            UWS_NUM=$((${UWS_MAJOR}*10000000000 + ${UWS_MINOR}*100000000 + ${UWS_MNT}*1000000 + ${UWS_SPEC}*10000 + ${UWS_BDVER}))
            addlog "CVC_NUM=${CVC_NUM}"
            addlog "UWS_NUM=${UWS_NUM}"
	    if [[ ${CVC_MAJOR} -gt 99 || ${CVC_MINOR} -gt 99 || ${CVC_MNT} -gt 99 || ${CVC_SPEC} -gt 99 || ${CVC_BDVER} -gt 9999 ]]; then
                addlog "Failed due to wrong current version format"
                rc=59
            fi
            if [[ ${UWS_MAJOR} -gt 99 || ${UWS_MINOR} -gt 99 || ${UWS_MNT} -gt 99 || ${UWS_SPEC} -gt 99 || ${UWS_BDVER} -gt 9999 ]]; then
                addlog "Failed due to wrong upgrade version format"
                rc=59
            fi

            DELTA=$((${CVC_NUM}-${UWS_NUM}))
            if [[ $DELTA -gt 0 ]]; then
              DELTA=$(( $DELTA/10000 ))
              if [ ! -f "$allow_DOWNLOAD" ]; then
                 if [[ $DELTA -eq 0 && ${CVC_SPEC} -eq ${UWS_SPEC} ]]; then
                   addlog "Failed due to current build number is bigger $CVC_BDVER > $UWS_BDVER"
                   rc=59
                 else
                   DELTA=$(( $DELTA/100 ))
                   if [[ $DELTA -eq 0 && ${CVC_MNT} -eq ${UWS_MNT} ]]; then
                     addlog "Failed due to current special version is bigger $CVC_SPEC > $UWS_SPEC"
                     rc=59
                   else
                     DELTA=$(( $DELTA/100 ))
                     if [[ $DELTA -eq 0 && ${CVC_MINOR} -eq ${UWS_MINOR} ]]; then
                       addlog "Failed due to current maintenance version is bigger $CVC_MNT > $UWS_MNT"
                       rc=59
                     else
                       DELTA=$(( $DELTA/100 ))
                       if [[ $DELTA -eq 0 && ${CVC_MAJOR} -eq ${UWS_MAJOR} ]]; then
                         addlog "Failed due to current minor version is bigger $CVC_MINOR > $UWS_MINOR"
                         rc=59
                       else
                         DELTA=$(( $DELTA/100 ))
                         if [[ $DELTA -eq 0 ]]; then
                           addlog "Failed due to current major version is bigger $CVC_MAJOR > $UWS_MAJOR"
                           rc=59
                         fi
                       fi
                     fi
                   fi
                 fi
              fi #end .allowdownload file does not exist
            fi
          fi
          if [[ $rc -eq 0 ]]; then
             if [ -f $APPS_DIR/ABE_release -a ! -d $UWS_REPO/"$UWS_MMVER-$UWS_BDVER"/IP500V2 ]; then
                ${cmd_MKDIR} -p $UWS_REPO/"$UWS_MMVER-$UWS_BDVER"/IP500V2
                ${cmd_UNZIP} -q $APPS_DIR/uws/UWS1000.zip -d $UWS_REPO/"$UWS_MMVER-$UWS_BDVER"/IP500V2
                if (($? != 0)); then
                   addlog "Failed to unzip archive!!!"
                   rc=52
                fi
             fi
         
             SCRIPT_PATH="/usr/lib/ipoffice"
             MOUNT_PATH="$mpont"
             #create repo for Linux Server
             if [ -e $SCRIPT_PATH/createUwsArchive.pl -a -e $SCRIPT_PATH/UpPackInv.xsd ]; then
                $SCRIPT_PATH/createUwsArchive.pl $SCRIPT_PATH/UpPackInv.xsd $MOUNT_PATH $UWS_REPO 2 "$CRTYPE"
                if [ $? -eq 0 ]; then
                   uwscleannam="$UWS_MMVER-$UWS_BDVER"
                   uwscleandir="${prepo}/*"
                   uwscleanful="${prepo}/$uwscleannam"
                   for lcdfile in $uwscleandir
                   do
                      if [[ -d "$lcdfile" ]]; then
                         if [[ "$lcdfile" != "$uwscleanful" ]]; then
                            $cmd_RM -rf "$lcdfile"
                         fi
                      fi
                   done
                fi
             else
                addlog "Failed to create file repository for Linux Server!!!"
                rc=53
             fi
          fi
    else
       rc=14
    fi
    umountcmd="${cmd_UMOUNT} ${mpont}"
    ${umountcmd}
    if [ $? -ne 0 ]; then
       addlog "Warning umount error ${umountcmd}"
    fi
    $cmd_RM -rf $mpont
    return $rc
}

####################### main ############################
# ./uwsfetch.sh prot://user:pass@host/remotedir [http://user:pass@host:port] 
# ex: ftp://aps:apspatch@47.135.152.80/isoimage/ipo9.2.iso
#########################################################

umask 002

if [[ $# -ne 1 && $# -ne 2 ]]; then
    echo 1>&2 Usage: protocol://username:password@host/remote_dir [proxy]
    adderrlog 1
    exit 1
fi

URI=$1
PROXYURL=""
LOCALDIR=/tmp/uwsfetch
LOCKFILE=/tmp/uwsfetchlock.txt

if [[ $# -eq 1 && $1 == "cancel" ]]; then
   ${cmd_GREP} Creating ${uwsfetch_LOGFILE}
   if [ "$?" -eq "0" ]; then
      addlog "cannot cancel due to creating repo data"
   else
      ${cmd_GREP} ' DVD\| USB' ${uwsfetch_LOGFILE}
      if [ "$?" -eq "0" ]; then
         addlog "cannot cancel due to DVD/USB operation"
      else
         oldpid=`ps aux | grep -F 'uwsfetch' | grep -v -F 'cancel' | grep -v -F 'grep' | awk '{ print $2 }'`
         if [ ! -z "$oldpid" ]; then
            $cmd_KILL -9 $oldpid
         fi
         adderrlog 13
         $cmd_RM -rf $LOCALDIR
         $cmd_RM -rf $LOCKFILE
      fi
   fi
   exit 0
fi

if [ $# -eq 2 ]; then
  PROXYURL=$2
fi

# create local dir
if [ ! -d $LOCALDIR ]; then
    $cmd_MKDIR $LOCALDIR
else
    $cmd_RM -f $LOCALDIR/*
fi

# reset log file
setlog

fn_parse_URI $URI protocol user_name password host remote_path port

# for file name
remote_path=`echo ${remote_path} | ${cmd_SED} 's/\/\//\//'`
file_name=$(basename $remote_path)
file_xtsn=${file_name##*.}
file_path=${remote_path%/*}
if [ -z "$file_path" ]; then
   file_path="/";
fi
# for relative path
afstr='/+'
if `echo "$remote_path" | $cmd_GREP "$afstr" > /dev/null 2>&1`
then
    remote_path=`echo "$remote_path" | $cmd_AWK '{print substr("'"$remote_path"'",3)}'`
    echo "relative remote path: $remote_path"
fi

# decode password for now
#password=`convert_sig_to_string ${password}`

#addlog "parse URI: $protocol://$user_name:$password@$host$remote_path $file_path/$file_name"

if [[ $protocol != "ftp" && $protocol != "sftp" && $protocol != "scp" &&
      $protocol != "http" && $protocol != "https" &&
      $protocol != "usb" && $protocol != "USB" &&
      $protocol != "local" && $protocol != "LOCAL" &&
      $protocol != "DVD" && $protocol != "dvd" ]]; then
   $cmd_RM -rf $LOCALDIR
   $cmd_RM -rf $LOCKFILE
   adderrlog 2
   exit 2
fi
 
if [[ $protocol != "usb" && $protocol != "USB" &&
      $protocol != "DVD" && $protocol != "dvd" ]]; then
   if [[ $file_xtsn != "iso" && $file_xtsn != "ISO" ]]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      adderrlog 3
      exit 3
   fi
   if [[ $protocol != "local" && $protocol != "LOCAL" ]]; then
      if [[ -z $PROXYURL ]]; then
         $cmd_PING -c 1 $host > /dev/null
         if [[ $? -ne 0 ]]; then
            $cmd_RM -rf $LOCALDIR
            $cmd_RM -rf $LOCKFILE
            adderrlog 4
            exit 4
         fi
      else
         fn_parse_URI $PROXYURL proxypro proxyusr proxypss proxyhst proxypth proxyprt
         $cmd_PING -c 1 $proxyhst > /dev/null
         if [[ $? -ne 0 ]]; then
            $cmd_RM -rf $LOCALDIR
            $cmd_RM -rf $LOCKFILE
            adderrlog 4
            exit 4
         fi
      fi
   fi
fi

if [ $protocol == "ftp" ]; then
   if [ -z "$user_name" ]; then
      user_name="anonymous"
   fi
   if [ -z "$password" ]; then
      password="anonymous"
   fi
   if [ ! -z $port ]; then
      $cmd_ECHO > /dev/tcp/${host}/${port}
      if [ $? -ne 0 ]; then
         $cmd_RM -rf $LOCALDIR
         $cmd_RM -rf $LOCKFILE
         adderrlog 12
         exit 12
      fi
      host="${host}:${port}"
   else
      $cmd_ECHO > /dev/tcp/${host}/21
      if [ $? -ne 0 ]; then
         $cmd_RM -rf $LOCALDIR
         $cmd_RM -rf $LOCKFILE
         adderrlog 12
         exit 12
      fi
   fi
   #
   #fn_ftp_download $host $user_name $password $file_path $file_name $LOCALDIR
   #
   fn_newftp_download $protocol $host $user_name $password $remote_path $file_path $file_name $LOCALDIR $PROXYURL
   rc=$?
   if [ $rc -ne 0 ]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      if [ $rc -eq 72 ]; then
         adderrlog 14
      else
         adderrlog 5
      fi
      exit 5
   fi
   addlog "Fetched ISO file from ftp server"
   fn_apply $file_name $LOCALDIR
   rc=$?
   if [ ${rc} -ne 0 ]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      if [ $rc -eq 59 ]; then
        adderrlog 11
      else
        adderrlog 6
      fi
      exit 6
   fi
fi

if [[ $protocol == "http" || $protocol == "https" ]]; then
   if [ -z "$user_name" ]; then
      user_name="anonymous"
   fi
   if [ -z "$password" ]; then
      password="anonymous"
   fi
   if [ ! -z $port ]; then
      $cmd_ECHO > /dev/tcp/${host}/${port}
      if [ $? -ne 0 ]; then
         $cmd_RM -rf $LOCALDIR
         $cmd_RM -rf $LOCKFILE
         adderrlog 12
         exit 12
      fi
      host="${host}:${port}"
   else
      $cmd_ECHO > /dev/tcp/${host}/80
      if [ $? -ne 0 ]; then
         $cmd_RM -rf $LOCALDIR
         $cmd_RM -rf $LOCKFILE
         adderrlog 12
         exit 12
      fi
   fi
   fn_http_download $protocol $host $user_name $password $remote_path $file_path $file_name $LOCALDIR $PROXYURL
   rc=$?
   if [ $rc -ne 0 ]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      if [ $rc -eq 12 ]; then
        adderrlog 14
      else
        adderrlog 7
      fi
      exit 7
   fi
   addlog "Fetched ISO file from http server"
   fn_apply $file_name $LOCALDIR
   rc=$?
   if [ ${rc} -ne 0 ]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      if [ $rc -eq 59 ]; then
        adderrlog 11
      else
        adderrlog 6
      fi
      exit 6
   fi
fi

if [[ $protocol == "sftp" || $protocol == "scp" ]]; then
   if [ -z "$user_name" ]; then
      user_name="anonymous"
   fi
   if [ -z "$password" ]; then
      password="anonymous"
   fi
   if [[ $protocol == "sftp" ]]; then
      fn_sftp_download $host $user_name $password $remote_path $LOCALDIR $file_name $port
   else
      fn_scp_download $host $user_name $password $remote_path $LOCALDIR $file_name $port
   fi
   rc=$?
   if [ $rc -ne 0 ]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      if [ $rc -eq 62 ]; then
        adderrlog 14
      else
        adderrlog 8
      fi
      exit 8
   fi
   addlog "Fetched ISO file from sftp server"
   fn_apply $file_name $LOCALDIR
   rc=$?
   if [ ${rc} -ne 0 ]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      if [ $rc -eq 59 ]; then
        adderrlog 11
      else
        adderrlog 6
      fi
      exit 6
   fi
fi

if [[ $protocol == "DVD" || $protocol == "dvd" ]]; then
   fn_dvd_download $remote_path $LOCALDIR file_name 
   rc=$?
   if [[ $rc -ne 0 && $rc -ne 1 ]]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      adderrlog 9
      exit 9
   fi
   addlog "Copied ISO file from DVD drive"
   fn_apply $file_name $LOCALDIR $rc
   rc=$?
   if [ $rc -ne 0 ]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      if [ $rc -eq 59 ]; then
        adderrlog 11
      else
        adderrlog 6
      fi
      exit 6
   fi
   $cmd_EJECT /dev/dvd
fi

if [[ $protocol == "usb" || $protocol == "USB" ]]; then
   fn_usb_download $remote_path $LOCALDIR file_name 
   rc=$?
   if [[ $rc -ne 0 && $rc -ne 1 ]]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      adderrlog 10
      exit 10
   fi
   addlog "Copied ISO file from USB drive"
   fn_apply $file_name $LOCALDIR $rc
   rc=$?
   if [ $rc -ne 0 ]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      if [ $rc -eq 59 ]; then
        adderrlog 11
      else
        adderrlog 6
      fi
      exit 6
   fi
fi

if [[ $protocol == "local" || $protocol == "LOCAL" ]]; then
   addlog "Access ISO file from local dir $remote_path"
    ${cmd_CHECKISO} ${remote_path}
    if [ $? -ne 0 ]; then
       addlog "Failed to validate ISO MD5 checksum: ${remote_path}"
       $cmd_RM -rf $LOCALDIR
       $cmd_RM -rf $LOCKFILE
       adderrlog 14
       exit 14
    fi
   fn_apply $file_name $file_path
   rc=$?
   if [ $rc -ne 0 ]; then
      $cmd_RM -rf $LOCALDIR
      $cmd_RM -rf $LOCKFILE
      if [ $rc -eq 59 ]; then
        adderrlog 11
      else
        adderrlog 6
      fi
      exit 6
   fi
fi

$cmd_RM -rf $LOCALDIR
$cmd_RM -rf $LOCKFILE

addlog "Succesfully created repo data for upgrade"
echo "<RepoDataStatus>Success</RepoDataStatus>" >> $uwsfetch_LOGFILE

exit 0
