#!/bin/bash

##################################################
#
#  protocol://user_name:passwd@host:port/file_path
# 
# Usage: http <http_server> <user_name> <password> <remote_dir> <local_dir>"
# download inventory files and generate meta data file
#
##################################################

g_NULL=/dev/null
declare -r cmd_DATE=/bin/date
declare -r cmd_FTP=/usr/bin/ftp
declare -r cmd_GREP=/bin/grep
declare -r cmd_RPM=/bin/rpm
declare -r cmd_CAT=/bin/cat
declare -r cmd_RM=/bin/rm
declare -r cmd_MKDIR=/bin/mkdir
declare -r cmd_WGET=/usr/bin/wget
declare -r cmd_AWK=/bin/awk
declare -r cmd_CUT=/bin/cut
declare -r cmd_SED=/bin/sed
declare -r cmd_MOUNT=/bin/mount
declare -r cmd_UMOUNT=/bin/umount
declare -r cmd_PERL=/usr/bin/perl
declare -r cmd_CP=/bin/cp
declare -r cmd_TOUCH=/bin/touch
declare -r cmd_UNZIP=/usr/bin/unzip
declare -r cmd_FIND=/bin/find
declare -r cmd_HEAD=/usr/bin/head
declare -r cmd_MKTEMP=/bin/mktemp
declare -r cmd_SCP=/usr/bin/scp
declare -r cmd_MD5SUM=/usr/bin/md5sum
declare -r cmd_PYTHON=/usr/bin/python
declare -r cmd_EJECT=/usr/sbin/eject
declare -r cmd_PING=/bin/ping
declare -r cmd_CURL=/usr/bin/curl
declare -r cmd_DMIDECODE=/usr/sbin/dmidecode

declare -r PSGSPASS=/usr/lib/ipoffice/googlestorageget.py

urldecode() {
    local data=${1//+/ }
    printf '%b' "${data//%/\x}"
}

urlencode()
{
echo $1 | $cmd_AWK '
    BEGIN {
	# We assume an awk implementation that is just plain dumb.
	# We will convert an character to its ASCII value with the
	# table ord[], and produce two-digit hexadecimal output
	# without the printf("%02X") feature.

	EOL = "%0A"		# "end of line" string (encoded)
	split ("1 2 3 4 5 6 7 8 9 A B C D E F", hextab, " ")
	hextab [0] = 0
	for ( i=1; i<=255; ++i ) ord [ sprintf ("%c", i) "" ] = i + 0
	if ("'"$EncodeEOL"'" == "yes") EncodeEOL = 1; else EncodeEOL = 0
    }
    {
	encoded = ""
	for ( i=1; i<=length ($0); ++i ) {
	    c = substr ($0, i, 1)
	    if ( c ~ /[a-zA-Z0-9.-]/ ) {
		encoded = encoded c		# safe character
	    } else if ( c == " " ) {
		encoded = encoded "+"	# special handling
	    } else {
		# unsafe character, encode it as a two-digit hex-number
		lo = ord [c] % 16
		hi = int (ord [c] / 16);
		encoded = encoded "%" hextab [hi] hextab [lo]
	    }
	}
	if ( EncodeEOL ) {
	    printf ("%s", encoded EOL)
	} else {
	    print encoded
	}
    }
    END {
    	#if ( EncodeEOL ) print ""
    }
'
#' "$@"
}

shlencode()
{
echo $1 | $cmd_AWK '
    BEGIN {
	# We assume an awk implementation that is just plain dumb.
	# We will convert an character to its ASCII value with the
	# table ord[], and produce two-digit hexadecimal output
	# without the printf("%02X") feature.

	EOL = "%0A"		# "end of line" string (encoded)
	split ("1 2 3 4 5 6 7 8 9 A B C D E F", hextab, " ")
	hextab [0] = 0
	for ( i=1; i<=255; ++i ) ord [ sprintf ("%c", i) "" ] = i + 0
	if ("'"$EncodeEOL"'" == "yes") EncodeEOL = 1; else EncodeEOL = 0
    }
    {
	encoded = ""
	for ( i=1; i<=length ($0); ++i ) {
	    c = substr ($0, i, 1)
	    if ( c ~ /[a-zA-Z0-9.-]/ ) {
		encoded = encoded c		# safe character
	    } else if ( c == " " ) {
		encoded = encoded "\\ "	# special handling
	    } else {
		# unsafe character, encode it as a two-digit hex-number
		#lo = ord [c] % 16
		#hi = int (ord [c] / 16);
		#encoded = encoded "%" hextab [hi] hextab [lo]
		encoded = encoded "\\" c
	    }
	}
	if ( EncodeEOL ) {
	    printf ("%s", encoded EOL)
	} else {
	    print encoded
	}
    }
    END {
    	#if ( EncodeEOL ) print ""
    }
'
#' "$@"
}

################################################################################
### Purpose: convert base64 encoded string 
### 
### Args:
###          $1 encoded string 
### Returns: decodded string

convert_sig_to_string()
{
  local INPUT_STR=`echo -n $1 | /usr/bin/base64 -d`
  if [ ! "$?" = "0" ] ; then
    echo -n "FAIL"
  fi
  echo -n $INPUT_STR
}

################################################################################
### Purpose: add a log entry into the file /opt/Avaya/uws_logs/uwsgetmeta.log
### 
### Args:
###          $1 log string 
### Returns: 

addlog()
{
  local timestamp=$( date '+%Y-%m-%dT%H:%M:%S' )
  local logstr=$1
  local logfile=/opt/Avaya/uws_logs/uwsgetmeta.log
  echo "<log>$timestamp $logstr</log>" | ${cmd_SED} 's/$/&\n/' >> $logfile
}

setlog()
{
  local logfile=/opt/Avaya/uws_logs/uwsgetmeta.log
  if [ -f "${logfile}" ]; then
      ${cmd_RM} -f ${logfile}
      ${cmd_TOUCH} ${logfile}
  fi
}


################################################################################
### Purpose: extract substring between prefix and suffix.
### 
### Args:
###          $1 string to extract from 
###          $2 prefix
###          $3 suffix
###
### Returns: substring

fn_parse_URI()
{
    local -r URI=$1
    local -r _arg_passback_protocol=$2
    local -r _arg_passback_username=$3
    local -r _arg_passback_password=$4
    local -r _arg_passback_host=$5
    local -r _arg_passback_remotepath=$6
    local -r _arg_passback_port=$7

    # parse URI to get protocol
    eval "${_arg_passback_protocol}=${URI%"://"*}"

    # parse URI to get remote path
    prefix=${URI%"/"*}
    eval "${_arg_passback_remotepath}=${URI#${prefix}"/"}"

    # parse URI to get user info if existing
    echo $URI | $cmd_GREP "@" > $g_NULL
    if [ $? -eq 0 ]; then
      # if URI contains @, then parse username and password
      noprefix=${URI#${URI%"://"*}"://"}
      user_info=${noprefix%"@"*}
      eval "${_arg_passback_username}=${user_info%":"*}"
      eval "${_arg_passback_password}=${user_info#*":"}"

      # get host
      noprefix=${URI#${URI%"@"*}"@"}
      host_info=${noprefix%%/*}
      echo $host_info | $cmd_GREP ":" > $g_NULL
      if [ $? -eq 0 ]; then
        eval "${_arg_passback_host}=${host_info%":"*}"
        eval "${_arg_passback_port}=${host_info#*":"}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      else
        eval "${_arg_passback_host}=${host_info#*"@"}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      fi
    else
      noprefix=${URI#${URI%"://"*}"://"}
      user_info=""
      eval "${_arg_passback_username}=${user_info%":"*}"
      eval "${_arg_passback_password}=${user_info#*":"}"
      host_info=${noprefix%%/*}
      echo $host_info | $cmd_GREP ":" > $g_NULL
      if [ $? -eq 0 ]; then
        eval "${_arg_passback_host}=${host_info%":"*}"
        eval "${_arg_passback_port}=${host_info#*":"}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      else
        eval "${_arg_passback_host}=${noprefix%%/*}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      fi
    fi
    return 0
}

################################################################################
### Purpose: validate MD5 checksum 
### Args:
###          $1 file to be validated
###          $2 MD5 checksum file
### Returns: substring

fn_validate_checksum()
{
    local -r FILEDATA=$1
    local -r FILEMD5S=$2
    if [[ ! -e "$FILEDATA" ]]; then
       return 1;
    fi
    if [[ ! -e "$FILEMD5S" ]]; then
       return 2;
    fi
    newmd5sum=`${cmd_MD5SUM} ${FILEDATA} | ${cmd_AWK} '{print $1}'`
    oldmd5sum=`${cmd_CAT} ${FILEMD5S} | ${cmd_HEAD} -n 1 | ${cmd_AWK} '{print $1}'`
    if [ "$newmd5sum" != "$oldmd5sum" ]; then
       return 3;
    fi
    return 0;
}

################################################################################
### Purpose: download the file 
###          and save in local directory
### Args:
###          $1 protocol
###          $2 host
###          $3 user_name
###          $4 password
###          $5 remote_path
###          $6 invfilename
###          $7 local_dir
###          $8 proxyurl
###
### Returns: substring

fn_http_download()
{
    local -r PROTOCOL=$1
    local -r HOST=$2
    local -r USERNM=$3
    local -r PASSWD=$4
    local -r REMOTEDIR=$5
    local -r INVFILE=$6
    local -r LOCALDIR=$7
    local -r PROXYSET=$8
    
    local -r PSGSPASS=/usr/lib/ipoffice/googlestorageget.py

    OHOST="${HOST%:*}"
    FILELISTFILE=/tmp/uwsfilelist_${OHOST}.txt
    if [ -e "$FILELISTFILE" ]; then
       ${cmd_GREP} "${INVFILE}" ${FILELISTFILE}
       if [[ $? -ne 0 ]]; then
          addlog "Failed to find: ${INVFILE} in file list $FILELISTFILE"
          return 11
       fi
    fi

    LASTONE=${REMOTEDIR: -1:1}
    if [ $USERNM == "anonymous" ]; then
       if [[ $REMOTEDIR == "/" || $LASTONE == "/" ]]; then
          URI=$PROTOCOL://${HOST}${REMOTEDIR}${INVFILE}
       else
          URI=$PROTOCOL://${HOST}${REMOTEDIR}/${INVFILE}
       fi
    else
       if [[ $REMOTEDIR == "/" || $LASTONE == "/" ]]; then
          URI=$PROTOCOL://$USERNM:$PASSWD@${HOST}${REMOTEDIR}${INVFILE}
       else
          URI=$PROTOCOL://$USERNM:$PASSWD@${HOST}${REMOTEDIR}/${INVFILE}
       fi
    fi
    # set proxy
    if [ -n "$PROXYSET" ]; then
       export http_proxy="$PROXYSET"
       addlog "setting $exportcmd"
    else
       unset http_proxy
    fi
    addlog "Retrieving the ${INVFILE} from ${URI}"
    platform="$(${cmd_DMIDECODE} "-s" "bios-vendor")"
    if [[ $platform == "Google" ]]; then
        prefix="\/gs"
        URIGS=${REMOTEDIR}${INVFILE}
        if [[ "$URIGS" == $prefix\/* ]]; then
            # Google Storage location
            NEWURI=${URIGS#$prefix}
            addlog "NEWURI = ${NEWURI}"
            URI=${NEWURI}
            ${cmd_PYTHON} ${PSGSPASS} "${URI}" "${LOCALDIR}/${INVFILE}"
            rc=$?
        else
            # get file with wget
            $cmd_WGET --no-check-certificate $URI -O ${LOCALDIR}/${INVFILE}
            rc=$?
        fi
    else
        # get file with wget
        $cmd_WGET --no-check-certificate $URI -O ${LOCALDIR}/${INVFILE}
        rc=$?
    fi
    if [ $rc -ne 0 ]; then
       addlog "$rc Failed to retrieve from http server: ${URI}"
       return 11
    fi
    return 0
}

################################################################################
### Purpose: ftp download the file using wget
###          and save in local directory
### Args:
###          $1 protocol
###          $2 host
###          $3 user_name
###          $4 password
###          $5 remote_path
###          $6 inv_file_name
###          $7 local_dir
###          $8 proxyurl
###
### Returns: substring

fn_newftp_download()
{
    local -r PROTOCOL=$1
    local -r HOST=$2
    local -r USERNM=$3
    local -r PASSWD=$4
    local -r REMOTEDIR=$5
    local -r INVFILE=$6
    local -r LOCALDIR=$7
    local -r PROXYSET=$8
    
    EUSERNM=${USERNM}
    DPASSWD="$(urldecode $PASSWD)"
    OHOST="${HOST%:*}"
    OPORT="${HOST#*:}"
    if [[ -z "$OPORT" || "$OPORT" == "$OHOST" ]]; then
       OPORT="21"
    fi
    FILELISTFILE=/tmp/uwsfilelist_${OHOST}.txt
    if [ -e "$FILELISTFILE" ]; then
       ${cmd_GREP} "${INVFILE}" ${FILELISTFILE}
       if [[ $? -ne 0 ]]; then
          addlog "Failed to find: ${INVFILE} in file list $FILELISTFILE"
          return 71
       fi
    fi
    
    LASTONE=${REMOTEDIR: -1:1}
    if [[ $REMOTEDIR == "/" || $LASTONE == "/" ]]; then
       URI=$PROTOCOL://${HOST}${REMOTEDIR}${INVFILE}
    else
       URI=$PROTOCOL://${HOST}${REMOTEDIR}/${INVFILE}
    fi
    # set proxy
    if [ -n "$PROXYSET" ]; then
       export http_proxy="$PROXYSET"
       addlog "setting $exportcmd"
    else
       unset http_proxy
    fi
    # get file with wget
    addlog "Retrieving the ${INVFILE} from ${URI}"
    $cmd_WGET --ftp-user=${USERNM} --ftp-password=${DPASSWD} $URI -O ${LOCALDIR}/${INVFILE}
    rc=$?
    if [ $rc -ne 0 ]; then
       addlog "$rc Failed to retrieve from ftp server: ${URI}"
       return 71
    fi
    return 0
}

################################################################################
### Purpose: download file from sftp server
###          and save in current local directory
### Args:
###          $1 host
###          $2 user_name
###          $3 password
###          $4 remote_path
###          $5 invfile
###          $6 local_dir
###
### Returns: substring

fn_sftp_download()
{
    local -r HOST=$1
    local -r USERNM=$2
    local -r PASSWD=$3
    local -r REMOTEDIR=$4
    local -r INVFILE=$5
    local -r LOCALDIR=$6

    local -r PSSHPASS=/usr/lib/ipoffice/psshsftp.py

    DPASSWD="$(urldecode $PASSWD)"
    OHOST="${HOST%:*}"
    OPORT="${HOST#*:}"
    if [[ -z "$OPORT" || "$OPORT" == "$OHOST" ]]; then
       OPORT="22"
    fi
    FILELISTFILE=/tmp/uwsfilelist_${OHOST}.txt
    if [ -e "$FILELISTFILE" ]; then
       ${cmd_GREP} "${INVFILE}" ${FILELISTFILE}
       if [[ $? -ne 0 ]]; then
          addlog "Failed to find: ${INVFILE} in file list $FILELISTFILE"
          return 61
       fi
    fi
    # command
    LASTONE=${REMOTEDIR: -1:1}
    if [[ $REMOTEDIR == "/" || $LASTONE == "/" ]]; then
       URI=${USERNM}@${OHOST}:${REMOTEDIR}${INVFILE}
    else
       URI=${USERNM}@${OHOST}:${REMOTEDIR}/${INVFILE}
    fi
    addlog "Retrieving ${INVFILE} from ${OHOST}:${REMOTEDIR}"
    #${cmd_PYTHON} ${PSSHPASS} "${DPASSWD}" -o "StrictHostKeyChecking no" -o "NumberOfPasswordPrompts=1" -o "ConnectTimeout=3" -v -P ${OPORT} ${URI} ${LOCALDIR}/${INVFILE}
    ${cmd_PYTHON} ${PSSHPASS} ${OHOST} ${OPORT} "${USERNM}" "${DPASSWD}" get "${REMOTEDIR}/${INVFILE}" "${LOCALDIR}/${INVFILE}"
    if [ $? -ne 0 ]; then
       addlog "Failed to sftp the file: ${INVFILE}"
       return 61
    fi
    return 0
}

################################################################################
### Purpose: download file from scp server
###          and save in current local directory
### Args:
###          $1 host
###          $2 user_name
###          $3 password
###          $4 remote_path
###          $5 invfile
###          $6 local_dir
###
### Returns: substring

fn_scp_download()
{
    local -r HOST=$1
    local -r USERNM=$2
    local -r PASSWD=$3
    local -r REMOTEDIR=$4
    local -r INVFILE=$5
    local -r LOCALDIR=$6

    local -r PSSHPASS=/usr/lib/ipoffice/psshpass.py

    DPASSWD="$(urldecode $PASSWD)"
    OHOST="${HOST%:*}"
    OPORT="${HOST#*:}"
    if [[ -z "$OPORT" || "$OPORT" == "$OHOST" ]]; then
       OPORT="22"
    fi
    FILELISTFILE=/tmp/uwsfilelist_${OHOST}.txt
    if [ -e "$FILELISTFILE" ]; then
       ${cmd_GREP} "${INVFILE}" ${FILELISTFILE}
       if [[ $? -ne 0 ]]; then
          addlog "Failed to find: ${INVFILE} in file list $FILELISTFILE"
          return 61
       fi
    fi
    # command
    LASTONE=${REMOTEDIR: -1:1}
    if [[ $REMOTEDIR == "/" || $LASTONE == "/" ]]; then
       URI=${USERNM}@${OHOST}:${REMOTEDIR}${INVFILE}
    else
       URI=${USERNM}@${OHOST}:${REMOTEDIR}/${INVFILE}
    fi
    addlog "Retrieving ${INVFILE} from ${OHOST}:${REMOTEDIR}"
    ${cmd_PYTHON} ${PSSHPASS} "${DPASSWD}" -o "StrictHostKeyChecking no" -o "NumberOfPasswordPrompts=1" -o "ConnectTimeout=3" -v -P ${OPORT} ${URI} ${LOCALDIR}/${INVFILE}
    if [ $? -ne 0 ]; then
       addlog "Failed to scp the file: ${INVFILE}"
       return 61
    fi
    return 0
}

################################################################################
### Purpose: extract tag values
### Args:
###          $1 file name
###          $2 localdir
###          $3 outputfile
###          $4 applname
###
### Returns: substring

fn_extract()
{
    FILENM=$1
    LCLDIR=$2
    FOUTNM=$3
    APPLNM=$4
    #extract
    NM=$LCLDIR/$FILENM
    appl=`$cmd_GREP '<applName' $NM | $cmd_CUT -d '>' -f 2 | $cmd_CUT -d '<' -f 1`
    node=`$cmd_GREP '<nodeName' $NM | $cmd_CUT -d '>' -f 2 | $cmd_CUT -d '<' -f 1`
    time=`$cmd_GREP '<invDtTm' $NM | $cmd_CUT -d '>' -f 2 | $cmd_CUT -d '<' -f 1`
    version=`$cmd_GREP '<version' $NM | $cmd_CUT -d '>' -f 2 | $cmd_CUT -d '<' -f 1`
    misc=`$cmd_GREP '<misc' $NM | $cmd_CUT -d '>' -f 2 | $cmd_CUT -d '<' -f 1`
    label=`$cmd_GREP '<label' $NM | $cmd_CUT -d '>' -f 2 | $cmd_CUT -d '<' -f 1`
    btype=`$cmd_GREP '<backupType' $NM | $cmd_CUT -d '>' -f 2 | $cmd_CUT -d '<' -f 1`
    itime=`$cmd_GREP '<invTimeStamp' $NM | $cmd_CUT -d '>' -f 2 | $cmd_CUT -d '<' -f 1`
    #record
    echo "  <comp name=\"$APPLNM\">" >> $FOUTNM
    echo "    <applName>$appl</applName>" >> $FOUTNM
    echo "    <time>$time</time>" >> $FOUTNM
    echo "    <version>$version</version>" >> $FOUTNM
    echo "    <misc>$misc</misc>" >> $FOUTNM
    echo "    <label>$label</label>" >> $FOUTNM
    echo "    <type>$btype</type>" >> $FOUTNM
    echo "    <invTime>$itime</invTime>" >> $FOUTNM
    echo "  </comp>" >> $FOUTNM
    
    return 0
}

################################################################################
### Purpose: write metadata header
### Args:
###          $1 file name
###
### Returns: substring

fn_writeheader()
{
    FOUTNM=$1
    if [ -e "$FOUTNM" ]; then
       $cmd_RM -f $FOUTNM
    fi
    $cmd_TOUCH $FOUTNM
    #record
    #echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" >> $FOUTNM
    #echo "<response status=\"1\">" >> $FOUTNM
    #echo "<data>" >> $FOUTNM
    echo "<ws_object>" >> $FOUTNM
    echo "<ipo_object>" >> $FOUTNM
    echo "<upgrade>" >> $FOUTNM
    echo "<backupMetadata>" >> $FOUTNM
    return 0
}

fn_writefoot()
{
    FOUTNM=$1
    echo "</backupMetadata>" >> $FOUTNM
    echo "</upgrade>" >> $FOUTNM
    echo "</ipo_object>" >> $FOUTNM
    echo "</ws_object>" >> $FOUTNM
    #echo "</data>" >> $FOUTNM
    #echo "</response>" >> $FOUTNM
    return 0
}

####################### main ############################
# ./uwsgetmeta.sh mch1:mch2 idx1:idx2 prot://user:pass@host/remotedir 
# ex: ftp://aps:apspatch@47.135.152.80/backupdir
#########################################################

umask 002

if [[ $# -ne 3 && $# -ne 4 ]]; then
    echo 1>&2 Usage: machines indexes protocol://username:password@host/remote_dir [proxy]
    exit 27
fi

MCHSTR=$1
IDXSTR=$2
URI=$3
PROXYURL=""
LOCALDIR=/tmp/uwsgetmeta

if [ $# -eq 4 ]; then
  PROXYURL=$4
fi

# create local dir
if [ ! -d $LOCALDIR ]; then
    $cmd_MKDIR $LOCALDIR
else
    $cmd_RM -fr $LOCALDIR/*
fi

# reset log file
setlog

FOUTNM="/opt/Avaya/uws_logs/uwsmetadata.xml"
if [ -f "${FOUTNM}" ]; then
   ${cmd_RM} -f ${FOUTNM}
   ${cmd_TOUCH} ${FOUTNM}
fi

LOCKFILE=/tmp/uwsgetmetalock.txt

fn_parse_URI $URI protocol user_name password host remote_path port

# for file name
remote_path=`echo ${remote_path} | ${cmd_SED} 's/\/\//\//'`
file_name=$(basename $remote_path)
file_xtsn=${file_name##*.}
file_path=${remote_path%/*}
if [ -z "$file_path" ]; then
   file_path="/";
fi
# for relative path
afstr='/+'
if `echo "$remote_path" | $cmd_GREP "$afstr" > /dev/null 2>&1`
then
    remote_path=`echo "$remote_path" | $cmd_AWK '{print substr("'"$remote_path"'",3)}'`
    echo "relative remote path: $remote_path"
fi

if [[ $protocol != "ftp" && $protocol != "sftp" && $protocol != "scp" &&
      $protocol != "http" && $protocol != "https" ]]; then
   addlog "Wrong protocol: $protcol"
   $cmd_RM -rf $LOCKFILE
   exit 1
fi
 
nhost=$host

if [[ -z $PROXYURL ]]; then
   $cmd_PING -c 1 $nhost > /dev/null
   if [[ $? -ne 0 ]]; then
      addlog "Failed to ping $nhost..."
   fi
else
   fn_parse_URI $PROXYURL proxypro proxyusr proxypss proxyhst proxypth proxyprt
   $cmd_PING -c 1 $proxyhst > /dev/null
   if [[ $? -ne 0 ]]; then
      addlog "Failed to ping $proxyhst..."
   fi
fi

if [ -z "$user_name" ]; then
   user_name="anonymous"
fi
if [ -z "$password" ]; then
   password="anonymous"
fi
if [ ! -z $port ]; then
   nhost="${host}:${port}"
fi

FILELISTFILE=/tmp/uwsfilelist_${host}.txt
if [[ $protocol == "sftp" || $protocol == "scp" ]]; then
   PSFTPLIST=/usr/lib/ipoffice/psshsftplist.py
   ${cmd_PYTHON} ${PSFTPLIST} ${host} ${port} "${user_name}" "${password}" ls "${remote_path}/*.xml" "${FILELISTFILE}"
   if [[ $? -eq 0 ]]; then
      addlog "get file list for $protocol:$host$remote_path in $FILELISTFILE"
   else
      $cmd_RM $FILELISTFILE
   fi
fi
if [[ $protocol == "http" || $protocol == "https" ]]; then
   lastchar=${remote_path#${remote_path%?}}
   phpscript="listxmlfiles.php"
   if [ $lastchar == "/" ]; then
      TMPURI=$protocol://$user_name:$password@${host}:${port}${remote_path}${phpscript}
   else
      TMPURI=$protocol://$user_name:$password@${host}:${port}${remote_path}/${phpscript}
   fi
   $cmd_WGET --no-check-certificate $TMPURI -O ${FILELISTFILE}
   if [[ $? -eq 0 ]]; then
      addlog "get file list for $protocol:$host$remote_path in $FILELISTFILE"
   else
      $cmd_RM $FILELISTFILE
   fi
fi
if [[ $protocol == "ftp" ]]; then
   lastchar=${remote_path#${remote_path%?}}
   if [ $lastchar == "/" ]; then
      TMPURI=$protocol://$user_name:$password@${host}:${port}${remote_path}
   else
      TMPURI=$protocol://$user_name:$password@${host}:${port}${remote_path}/
   fi
   $cmd_CURL --list-only -s ${TMPURI} | $cmd_GREP ".xml" > ${FILELISTFILE}
   if [[ $? -eq 0 ]]; then
      addlog "get file list for $protocol:$host$remote_path in $FILELISTFILE"
   else
      $cmd_RM $FILELISTFILE
   fi
fi

#FOUTNM=$LOCALDIR/uwsmetadata.xml
FOUTNM="/opt/Avaya/uws_logs/uwsmetadata.xml"
fn_writeheader $FOUTNM

rcode=0

# for index
IDXARR=${IDXSTR//:/ }
for idx in $IDXARR
do
   if [[ ${#idx} < 2 ]]; then
      nid="0${idx}"
   else
      nid="${idx}"
   fi
   echo "<index name=\"$nid\">" >> $FOUTNM
   # for nodes
   MCHARR=${MCHSTR//:/ }
   for MCH in $MCHARR
   do
      echo "<node name=\"$MCH\">" >> $FOUTNM
      # for app name: ipo linux onex vmpro acripo
      for app in ipo linux onex vmpro acripo webrtcgw
      do
         invfilename="${nid}_${MCH}+${app}_bckp.xml"
         if [ $protocol == "ftp" ]; then
            if [ ! -z $port ]; then
               nhost="${host}:${port}"
            fi
            fn_newftp_download $protocol $nhost $user_name $password $remote_path $invfilename $LOCALDIR $PROXYURL
            rc=$?
            if [ $rc -ne 0 ]; then
               addlog "continue to the next"
               rcode=$rc
               continue
            fi
            addlog "Got ${invfilename} from ftp server"
         fi
         if [[ $protocol == "http" || $protocol == "https" ]]; then
            fn_http_download $protocol $nhost $user_name $password $remote_path $invfilename $LOCALDIR $PROXYURL
            rc=$?
            if [ $rc -ne 0 ]; then
               addlog "continue to the next"
               rcode=$rc
               continue
            fi
            addlog "Got ${invfilename} from http server"
         fi
         if [[ $protocol == "sftp" ]]; then
            fn_sftp_download $nhost $user_name $password $remote_path $invfilename $LOCALDIR
            rc=$?
            if [ $rc -ne 0 ]; then
               addlog "continue to the next"
               rcode=$rc
               continue
            fi
            addlog "Got ${invfilename} from sftp server"
         fi
         if [[ $protocol == "scp" ]]; then
            fn_scp_download $nhost $user_name $password $remote_path $invfilename $LOCALDIR
            rc=$?
            if [ $rc -ne 0 ]; then
               addlog "continue to the next"
               rcode=$rc
               continue
            fi
            addlog "Got ${invfilename} from scp server"
         fi
         if [ -e "$LOCALDIR/$invfilename" ]; then
            fn_extract $invfilename $LOCALDIR $FOUTNM $app
            rc=$?
            $cmd_RM -f "$LOCALDIR/$invfilename"
            if [ ${rc} -ne 0 ]; then
               addlog "Failed to extract data from $LOCALDIR/$invfilename"
               rcode=$rc
               continue
            fi
         fi
      done
      echo "</node>" >> $FOUTNM
   done
   echo "</index>" >> $FOUTNM
done


fn_writefoot $FOUTNM

if [ $rcode -eq 0 ]; then
   addlog "Succesfully created metadata"
fi

$cmd_RM $FILELISTFILE
$cmd_RM -rf $LOCKFILE
$cmd_RM -rf $LOCALDIR

exit 0
