#!/bin/bash

##################################################
#
#  local_file protocol://user_name:passwd@host:port/file_path 
# 
#  push a local file onto a remote server
#
##################################################

g_NULL=/dev/null
declare -r cmd_DATE=/bin/date
declare -r cmd_FTP=/usr/bin/ftp
declare -r cmd_GREP=/bin/grep
declare -r cmd_RPM=/bin/rpm
declare -r cmd_CAT=/bin/cat
declare -r cmd_RM=/bin/rm
declare -r cmd_MKDIR=/bin/mkdir
declare -r cmd_WGET=/usr/bin/wget
declare -r cmd_AWK=/bin/awk
declare -r cmd_CUT=/bin/cut
declare -r cmd_SED=/bin/sed
declare -r cmd_MOUNT=/bin/mount
declare -r cmd_UMOUNT=/bin/umount
declare -r cmd_PERL=/usr/bin/perl
declare -r cmd_CP=/bin/cp
declare -r cmd_TOUCH=/bin/touch
declare -r cmd_UNZIP=/usr/bin/unzip
declare -r cmd_FIND=/bin/find
declare -r cmd_HEAD=/usr/bin/head
declare -r cmd_MKTEMP=/bin/mktemp
declare -r cmd_SCP=/usr/bin/scp
declare -r cmd_MD5SUM=/usr/bin/md5sum
declare -r cmd_PYTHON=/usr/bin/python
declare -r cmd_EJECT=/usr/sbin/eject
declare -r cmd_CURL=/usr/bin/curl
declare -r cmd_PING=/bin/ping
declare -r cmd_TR=/usr/bin/tr
declare -r cmd_MV=/bin/mv
declare -r cmd_DMIDECODE=/usr/sbin/dmidecode

declare -r PSGSPASS=/usr/lib/ipoffice/googlestorageput.py
declare -r PSGSPASSGET=/usr/lib/ipoffice/googlestorageget.py
declare -r PSGSPASSDEL=/usr/lib/ipoffice/googlestoragedel.py

UWSPUSHLOG=/opt/Avaya/uws_logs/uwspushfile.log

urldecode() {
    local data=${1//+/ }
    printf '%b' "${data//%/\x}"
}

urlencode()
{
echo $1 | $cmd_AWK '
    BEGIN {
      # We assume an awk implementation that is just plain dumb.
      # We will convert an character to its ASCII value with the
      # table ord[], and produce two-digit hexadecimal output
      # without the printf("%02X") feature.

      EOL = "%0A"     # "end of line" string (encoded)
      split ("1 2 3 4 5 6 7 8 9 A B C D E F", hextab, " ")
      hextab [0] = 0
      for ( i=1; i<=255; ++i ) ord [ sprintf ("%c", i) "" ] = i + 0
      if ("'"$EncodeEOL"'" == "yes") EncodeEOL = 1; else EncodeEOL = 0
    }
    {
      encoded = ""
      for ( i=1; i<=length ($0); ++i ) {
          c = substr ($0, i, 1)
          if ( c ~ /[a-zA-Z0-9.-]/ ) {
              encoded = encoded c     # safe character
          } else if ( c == " " ) {
              encoded = encoded "+"   # special handling
          } else {
              # unsafe character, encode it as a two-digit hex-number
              lo = ord [c] % 16
              hi = int (ord [c] / 16);
              encoded = encoded "%" hextab [hi] hextab [lo]
          }
      }
      if ( EncodeEOL ) {
          printf ("%s", encoded EOL)
      } else {
          print encoded
      }
    }
    END {
      #if ( EncodeEOL ) print ""
    }
'
#' "$@"
}

shlencode()
{
echo $1 | $cmd_AWK '
    BEGIN {
      # We assume an awk implementation that is just plain dumb.
      # We will convert an character to its ASCII value with the
      # table ord[], and produce two-digit hexadecimal output
      # without the printf("%02X") feature.

      EOL = "%0A"     # "end of line" string (encoded)
      split ("1 2 3 4 5 6 7 8 9 A B C D E F", hextab, " ")
      hextab [0] = 0
      for ( i=1; i<=255; ++i ) ord [ sprintf ("%c", i) "" ] = i + 0
      if ("'"$EncodeEOL"'" == "yes") EncodeEOL = 1; else EncodeEOL = 0
    }
    {
      encoded = ""
      for ( i=1; i<=length ($0); ++i ) {
          c = substr ($0, i, 1)
          if ( c ~ /[a-zA-Z0-9.-]/ ) {
              encoded = encoded c     # safe character
          } else if ( c == " " ) {
              encoded = encoded "\\ " # special handling
          } else {
              # unsafe character, encode it as a two-digit hex-number
              #lo = ord [c] % 16
              #hi = int (ord [c] / 16);
              #encoded = encoded "%" hextab [hi] hextab [lo]
              encoded = encoded "\\" c
          }
      }
      if ( EncodeEOL ) {
          printf ("%s", encoded EOL)
      } else {
          print encoded
      }
    }
    END {
      #if ( EncodeEOL ) print ""
    }
'
#' "$@"
}

################################################################################
### Purpose: convert base64 encoded string 
### 
### Args:
###          $1 encoded string 
### Returns: decodded string

convert_sig_to_string()
{
  local INPUT_STR=`echo -n $1 | /usr/bin/base64 -d`
  if [ ! "$?" = "0" ] ; then
    echo -n "FAIL"
  fi
  echo -n $INPUT_STR
}

################################################################################
### Purpose: add a log entry into the push log file file 
### 
### Args:
###          $1 log string 
### Returns: 

addlog()
{
  local timestamp=$( date '+%Y-%m-%dT%H:%M:%S' )
  local logstr=$1
  local logfile=$UWSPUSHLOG
  echo "<log>$timestamp $logstr</log>" | ${cmd_SED} 's/$/&\n/' >> $logfile
}

setlog()
{
  local logfile=$UWSPUSHLOG
  if [ -f "${logfile}" ]; then
      ${cmd_RM} -f ${logfile}
      ${cmd_TOUCH} ${logfile}
  fi
}


################################################################################
### Purpose: extract substring between prefix and suffix.
### 
### Args:
###          $1 string to extract from 
###          $2 prefix
###          $3 suffix
###
### Returns: substring

fn_parse_URI()
{
    local -r URI=$1
    local -r _arg_passback_protocol=$2
    local -r _arg_passback_username=$3
    local -r _arg_passback_password=$4
    local -r _arg_passback_host=$5
    local -r _arg_passback_remotepath=$6
    local -r _arg_passback_port=$7

    # parse URI to get protocol
    eval "${_arg_passback_protocol}=${URI%"://"*}"

    # parse URI to get remote path
    prefix=${URI%"/"*}
    eval "${_arg_passback_remotepath}=${URI#${prefix}"/"}"

    # parse URI to get user info if existing
    echo $URI | $cmd_GREP "@" > $g_NULL
    if [ $? -eq 0 ]; then
      # if URI contains @, then parse username and password
      noprefix=${URI#${URI%"://"*}"://"}
      user_info=${noprefix%"@"*}
      eval "${_arg_passback_username}=${user_info%":"*}"
      eval "${_arg_passback_password}=${user_info#*":"}"

      # get host
      noprefix=${URI#${URI%"@"*}"@"}
      host_info=${noprefix%%/*}
      echo $host_info | $cmd_GREP ":" > $g_NULL
      if [ $? -eq 0 ]; then
        eval "${_arg_passback_host}=${host_info%":"*}"
        eval "${_arg_passback_port}=${host_info#*":"}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      else
        eval "${_arg_passback_host}=${host_info#*"@"}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      fi
    else
      noprefix=${URI#${URI%"://"*}"://"}
      user_info=""
      eval "${_arg_passback_username}=${user_info%":"*}"
      eval "${_arg_passback_password}=${user_info#*":"}"
      host_info=${noprefix%%/*}
      echo $host_info | $cmd_GREP ":" > $g_NULL
      if [ $? -eq 0 ]; then
        eval "${_arg_passback_host}=${host_info%":"*}"
        eval "${_arg_passback_port}=${host_info#*":"}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      else
        eval "${_arg_passback_host}=${noprefix%%/*}"
        eval "${_arg_passback_remotepath}=${noprefix#$host_info}"
      fi
    fi
    return 0
}

################################################################################
### Purpose: validate MD5 checksum 
### Args:
###          $1 file to be validated
###          $2 MD5 checksum file
### Returns: substring

fn_validate_checksum()
{
    local -r FILEDATA=$1
    local -r FILEMD5S=$2
    if [[ ! -e "$FILEDATA" ]]; then
       return 1;
    fi
    if [[ ! -e "$FILEMD5S" ]]; then
       return 2;
    fi
    newmd5sum=`${cmd_MD5SUM} ${FILEDATA} | ${cmd_AWK} '{print $1}'`
    oldmd5sum=`${cmd_CAT} ${FILEMD5S} | ${cmd_HEAD} -n 1 | ${cmd_AWK} '{print $1}'`
    if [ "$newmd5sum" != "$oldmd5sum" ]; then
       return 3;
    fi
    return 0;
}

################################################################################
### Purpose: push the file onto a server
### Args:
###          $1 ispull
###          $2 protocol
###          $3 host
###          $4 user_name
###          $5 password
###          $6 remote_path
###          $7 local_file_dir
###          $8 proxyurl
###
### Returns: substring

fn_http_push()
{
    local -r ISPULL=$1
    local -r PROTOCOL=$2
    local -r HOST=$3
    local -r USERNM=$4
    local -r PASSWD=$5
    local -r REMOTEDIR=$6
    local -r LOCALDIR=$7
    local -r PROXYSET=$8
 
    # locafile name is urlencoded
    DLOCALDIR=`urldecode ${LOCALDIR}`

    # set proxy
    if [ -n "$PROXYSET" ]; then
       export http_proxy="$PROXYSET"
       addlog "setting $PROXYSET"
    else
       unset http_proxy
    fi
    platform="$(${cmd_DMIDECODE} "-s" "bios-vendor")"
    if [[ $ISPULL == "pull" ]]; then
       URI=$PROTOCOL://$USERNM:$PASSWD@${HOST}${REMOTEDIR}
       if [[ $platform == "Google" ]]; then
           prefix="\/gs"
           URIGS=${REMOTEDIR}${INVFILE}
           if [[ "$REMOTEDIR" == $prefix\/* ]]; then
               # Google Storage location.
               NEWREMOTEDIR=${REMOTEDIR#$prefix}
               URIGS=${NEWREMOTEDIR}${INVFILE}
               ${cmd_PYTHON} ${PSGSPASSGET} "${URIGS}" "${DLOCALDIR}"
               rc=$?
           else
               # Assume regular HTTP location.
               # get file with wget
               addlog "Retrieving ${URI}"
               $cmd_WGET --no-check-certificate $URI -O "${DLOCALDIR}"
               rc=$?
           fi
       else
           # get file with wget
           addlog "Retrieving ${URI}"
           $cmd_WGET --no-check-certificate $URI -O "${DLOCALDIR}"
           rc=$?
       fi
       if [ $rc -ne 0 ]; then
          addlog "Failed to retrieve from http server: ${URI}"
          return 11
       fi
    elif [[ $ISPULL == "push" ]]; then
       # assume password is urlencoded.
       EUSERNM="${USERNM}:${PASSWD}"
       # assume upload.php is placed at the remote dir for accepting file
       SCPTNAME="upload.php"
       FILENAME=$(basename $REMOTEDIR)
       FILEPATH=${REMOTEDIR%/*}
       LFILENAME=$(basename "$DLOCALDIR")
       LFILEPATH=${DLOCALDIR%/*}
       # rename
       if [ "$LFILENAME" == "$DLOCALDIR" ]; then
          DLNEWFILE=`urldecode $FILENAME`
       else
          DLNEWFILE=`urldecode $LFILEPATH/$FILENAME`
       fi
       if [ "$DLNEWFILE" != "$DLOCALDIR" ]; then
          $cmd_MV "$DLOCALDIR" "$DLNEWFILE"
          if [ $? -ne 0 ]; then
             addlog "Warning mv error: ${DLOCALDIR} to ${DLNEWFILE}"
          fi
       fi
       # url
       URI=$PROTOCOL://${HOST}${FILEPATH}/${SCPTNAME}
       # push file with curl
       NECODE="-k -sL -w \"%{http_code}\""
       addlog "Pushing ${DLOCALDIR} to ${REMOTEDIR}"

       RCODE=0
       if [[ $platform == "Google" ]]; then
           prefix="\/gs"
           if [[ "$REMOTEDIR" == $prefix\/* ]]; then
               # Google Storage location.
               NEWREMOTEDIR=${REMOTEDIR#$prefix}
               `${cmd_PYTHON} ${PSGSPASS} "${DLNEWFILE}" "${NEWREMOTEDIR}"`
               RCODE=$?
           else
               # Assume regular HTTP location.
               addlog "Pushing ${DLOCALDIR} to ${HOST}${REMOTEDIR}"
               RCODE=`$cmd_CURL ${NECODE} --user "${EUSERNM}" -F uploadedfile=@"${DLNEWFILE}" -X POST ${URI}`
           fi
       else
           addlog "Pushing ${DLOCALDIR} to ${HOST}${REMOTEDIR}"
           RCODE=`$cmd_CURL ${NECODE} --user "${EUSERNM}" -F uploadedfile=@"${DLNEWFILE}" -X POST ${URI}`
       fi
       # quota size exceeded case 
       if [[ -n `echo $RCODE | grep "failed"` ]] 
       then 
          addlog "Failed to push file to http server: ${URI}"
          return 11
       fi    
       RCODE=`echo $RCODE | $cmd_AWK '{print $NF}' | $cmd_TR -d '\"'`
       if [ "$DLNEWFILE" != "$DLOCALDIR" ]; then
          # rename back
          $cmd_MV "$DLNEWFILE" "$DLOCALDIR"
          if [ $? -ne 0 ]; then
             addlog "Warning mv error: ${DLOCALDIR} from ${DLNEWFILE}"
          fi
       fi
       if [[ $RCODE -ne 200 && $RCODE -ne 100 ]]; then
          addlog "Failed to push file to http server: ${URI}"
          return 11
       fi
       addlog "Pushed ${DLOCALDIR} to ${URI}"
    else
       # delete remote file
       # assume password is urlencoded.
       EUSERNM="${USERNM}:${PASSWD}"
       # assume delete.php is placed at the remote dir for deleting file
       SCPTNAME="delete.php"
       FILENAME=$(basename $REMOTEDIR)
       FILEPATH=${REMOTEDIR%/*}
       # url
       URI=$PROTOCOL://${HOST}${FILEPATH}/${SCPTNAME}?filename=${FILENAME}
       # delete file with curl for non-google storage locations
       NECODE="-k -sL -w \"%{http_code}\""
       NESINK="-o /dev/null"
       addlog "Deleting ${HOST}${REMOTEDIR}"
       RCODE=0
       if [[ $platform == "Google" ]]; then
           prefix="\/gs"
           if [[ "$REMOTEDIR" == $prefix\/* ]]; then
               # Google Storage location.
               NEWREMOTEDIR=${REMOTEDIR#$prefix}
               `${cmd_PYTHON} ${PSGSPASSDEL} "${NEWREMOTEDIR}"`
               RCODE=$?
           else 
               # Assume regular HTTP location.
               RCODE=`$cmd_CURL ${NECODE} --user "${EUSERNM}" -X POST ${URI} ${NESINK}`
               RCODE=`echo $RCODE | $cmd_TR -d '\"'`
           fi
       else 
           RCODE=`$cmd_CURL ${NECODE} --user "${EUSERNM}" -X POST ${URI} ${NESINK}`
           RCODE=`echo $RCODE | $cmd_TR -d '\"'`
       fi
       if [[ $RCODE -ne 200 && $RCODE -ne 100 ]]; then
          addlog "Failed to delete file at http server: ${URI}"
          return 11
       fi
       addlog "Deleted ${FILENAME} to ${URI}"
    fi
    return 0
}

################################################################################
### Purpose: ftp download the file using wget
###          and save in local directory
### Args:
###          $1 ispull
###          $2 protocol
###          $3 host
###          $4 user_name
###          $5 password
###          $6 remote_path
###          $7 local_dir
###          $8 proxyurl
###
### Returns: substring

fn_ftp_push()
{
    local -r ISPULL=$1
    local -r PROTOCOL=$2
    local -r HOST=$3
    local -r USERNM=$4
    local -r PASSWD=$5
    local -r REMOTEDIR=$6
    local -r LOCALDIR=$7
    local -r PROXYSET=$8
    
    # locafile name is urlencoded
    DLOCALDIR=`urldecode ${LOCALDIR}`

    URI=$PROTOCOL://${USERNM}:${PASSWD}@${HOST}${REMOTEDIR}
    # set proxy
    if [ -n "$PROXYSET" ]; then
       export http_proxy="$PROXYSET"
       addlog "setting $PROXYSET"
    else
       unset http_proxy
    fi
    if [[ $ISPULL == "pull" ]]; then
       # get file with wget
       addlog "Retrieving ${URI}"
       $cmd_WGET -t 2 --ftp-user=${USERNM} --ftp-password=${DPASSWD} $URI -O "${DLOCALDIR}"
       rc=$?
       if [ $rc -ne 0 ]; then
          addlog "$rc Failed to retrieve from ftp server: ${URI}"
          return 71
       fi
    elif [[ $ISPULL == "push" ]]; then
       # push file with curl
       addlog "Pushing file ${DLOCALDIR} to ${URI}"
       $cmd_CURL -T "${DLOCALDIR}" $URI
       if [ $? -ne 0 ]; then
          addlog "Failed to push to ftp server: ${URI}"
          return 71
       fi
       addlog "Pushed the file ${DLOCALDIR} to ${URI}"
    else
       # delete file with curl
       FILENAME=$(basename $REMOTEDIR)
       FILEPATH=${REMOTEDIR:1:${#REMOTEDIR}-1}
       DFILEPATH=`urldecode ${FILEPATH}`
       URIDEL=$PROTOCOL://${HOST}
       addlog "Deleting file ${FILEPATH} at ${URIDEL}"
       $cmd_CURL --user ${USERNM}:${PASSWD} -X "DELE ${DFILEPATH}" ${URIDEL}
       if [[ $? -ne 0 && $? -ne 19 ]]; then
          addlog "Failed to delete at ftp server: ${URIDEL}"
          return 71
       fi
       addlog "Deleted the file ${FILEPATH} at ${URIDEL}"
    fi
    return 0
}

################################################################################
### Purpose: download file from scp server
###          and save in current local directory
### Args:
###          $1 ispull
###          $2 host
###          $3 user_name
###          $4 password
###          $5 remote_path
###          $6 local_dir
###          $7 file_name
###          $8 port
###
### Returns: substring

fn_scp_push()
{
    local -r ISPULL=$1
    local -r HOST=$2
    local -r USERNM=$3
    local -r PASSWD=$4
    local -r REMOTEDIR=$5
    local -r LOCALDIR=$6
    local -r FILENM=$7
    local -r PORTN=$8

    local -r PSSHPASS=/usr/lib/ipoffice/psshpass.py

    # locafile name is urlencoded
    DLOCALDIR=`urldecode ${LOCALDIR}`
    #DLOCALDIR="$(echo $DLOCALDIR | sed -e 's/[ @()&]/\\&/g')"
    # remotedir is urlencoded
    DREMOTEDIR=`urldecode ${REMOTEDIR}`
    #DREMOTEDIR="$(echo $DREMOTEDIR | sed -e 's/[ @()&]/\\&/g')"

    DPASSWD="$(urldecode $PASSWD)"

    OPORT=$PORTN
    if [[ -z "$PORTN" ]]; then
       OPORT=22
    fi

    # command
    addlog "Copying ${HOST}:${DREMOTEDIR}"
    if [[ $ISPULL == "pull" ]]; then
       ${cmd_PYTHON} ${PSSHPASS} "${DPASSWD}" -o "StrictHostKeyChecking no" -o "NumberOfPasswordPrompts=1" -o "ConnectTimeout=5" -v -P ${OPORT} "${USERNM}@${HOST}:\"${DREMOTEDIR}\"" "${DLOCALDIR}"
       if [ $? -ne 0 ]; then
          addlog "Failed to scp the file: ${DLOCALDIR} from ${HOST}"
          return 61
       fi
    elif [[ $ISPULL == "push" ]]; then
       ${cmd_PYTHON} ${PSSHPASS} "${DPASSWD}" -o "StrictHostKeyChecking no" -o "NumberOfPasswordPrompts=1" -o "ConnectTimeout=5" -v -P ${OPORT} "${DLOCALDIR}" "${USERNM}@${HOST}:\"${DREMOTEDIR}\""
       if [ $? -ne 0 ]; then
          addlog "Failed to scp the file: ${DLOCALDIR} to ${HOST}"
          return 61
       fi
    else
       # delete
       PSSHPASSDEL=/usr/lib/ipoffice/psshpassdel.py
       ${cmd_PYTHON} ${PSSHPASSDEL} "${DPASSWD}" -o "StrictHostKeyChecking no" -o "NumberOfPasswordPrompts=1" -o "ConnectTimeout=5" -v "${USERNM}@${HOST}" -D ${OPORT} rm -rf \"${DREMOTEDIR}\"
       if [ $? -ne 0 ]; then
          addlog "Failed to delete the file: ${DREMOTEDIR} at ${HOST}"
          return 61
       fi
    fi
    return 0
}


################################################################################
### Purpose: download file from sftp server
###          and save in current local directory
### Args:
###          $1 ispull
###          $2 host
###          $3 user_name
###          $4 password
###          $5 remote_path
###          $6 local_dir
###          $7 file_name
###          $8 port
###
### Returns: substring

fn_sftp_push()
{
    local -r ISPULL=$1
    local -r HOST=$2
    local -r USERNM=$3
    local -r PASSWD=$4
    local -r REMOTEDIR=$5
    local -r LOCALDIR=$6
    local -r FILENM=$7
    local -r PORTN=$8

    local -r PSSHPASS=/usr/lib/ipoffice/psshsftp.py

    # locafile name is urlencoded
    DLOCALDIR=`urldecode ${LOCALDIR}`
    # remotedir is urlencoded
    DREMOTEDIR=`urldecode ${REMOTEDIR}`

    DPASSWD="$(urldecode $PASSWD)"

    OPORT=$PORTN
    if [[ -z "$PORTN" ]]; then
       OPORT=22
    fi

    # command
    addlog "Copying ${HOST}:${DREMOTEDIR}"
    if [[ $ISPULL == "pull" ]]; then
       ${cmd_PYTHON} ${PSSHPASS} ${HOST} ${OPORT} "${USERNM}" "${DPASSWD}" get "${DREMOTEDIR}" "${DLOCALDIR}"
       if [ $? -ne 0 ]; then
          addlog "Failed to sftp the file: ${DLOCALDIR} from ${HOST}/${DREMOTEDIR}"
          return 61
       fi
    elif [[ $ISPULL == "push" ]]; then
       ${cmd_PYTHON} ${PSSHPASS} ${HOST} ${OPORT} "${USERNM}" "${DPASSWD}" put "${DLOCALDIR}" "${DREMOTEDIR}"
       if [ $? -ne 0 ]; then
          addlog "Failed to sftp the file: ${DLOCALDIR} to ${HOST}/${DREMOTEDUR}"
          return 61
       fi
    else
       # delete
       ${cmd_PYTHON} ${PSSHPASS} ${HOST} ${OPORT} "${USERNM}" "${DPASSWD}" rm "${DREMOTEDIR}" null
       if [ $? -ne 0 ]; then
          addlog "Failed to delete the file: ${DREMOTEDIR} at ${HOST}"
          return 61
       fi
    fi
    return 0
}


####################### main ############################
# ./uwspush.sh pull|push|delete loc_file prot://user:pass@host/remotedir 
# ex: push /tmp/localfile ftp://aps:apspatch@47.135.152.80/backup/
#########################################################

umask 002

if [[ $# -ne 3 && $# -ne 4 ]]; then
    echo 1>&2 "Usage: [pull|push|delete] local_file  protocol://[username:password@]host[:port]/remote_dir [proxy]"
    exit 27
fi

ISPULL=$1
LOCALDIR=$2
URI=$3
PROXYURL=""

if [ $# -eq 4 ]; then
  PROXYURL=$4
fi

# reset log file
setlog

fn_parse_URI $URI protocol user_name password host remote_path port

# for file name
remote_path=`echo ${remote_path} | ${cmd_SED} 's/\/\//\//'`
file_name=$(basename $remote_path)
file_xtsn=${file_name##*.}
file_path=${remote_path%/*}
if [ -z "$file_path" ]; then
   file_path="/";
fi
# for relative path
afstr='/+'
if `echo "$remote_path" | $cmd_GREP "$afstr" > /dev/null 2>&1`
then
    remote_path=`echo "$remote_path" | $cmd_AWK '{print substr("'"$remote_path"'",3)}'`
    echo "relative remote path: $remote_path"
fi


if [[ $protocol != "ftp" && $protocol != "sftp" && $protocol != "scp" &&
      $protocol != "http" && $protocol != "https" ]]; then
   addlog "Wrong protocol: $protcol"
   exit 1
fi
 
if [[ -z $PROXYURL ]]; then
   $cmd_PING -c 1 $host > /dev/null
   if [[ $? -ne 0 ]]; then
      addlog "Failed to ping $host..."
   fi
else
   fn_parse_URI $PROXYURL proxypro proxyusr proxypss proxyhst proxypth proxyprt
   $cmd_PING -c 1 $proxyhst > /dev/null
   if [[ $? -ne 0 ]]; then
      addlog "Failed to ping $proxyhst..."
   fi
fi

if [ -z "$user_name" ]; then
   user_name="anonymous"
fi
if [ -z "$password" ]; then
   password="anonymous"
fi

if [ $protocol == "ftp" ]; then
   if [ ! -z $port ]; then
      $cmd_ECHO > /dev/tcp/${host}/${port}
      if [ $? -ne 0 ]; then
         addlog "Wrong port: $port, no connectivity."
         exit 1
      fi
      host="${host}:${port}"
   else
      $cmd_ECHO > /dev/tcp/${host}/21
      if [ $? -ne 0 ]; then
         addlog "No connectivity to default ftp port 21."
         exit 1
      fi
   fi
   fn_ftp_push $ISPULL $protocol $host $user_name $password $remote_path $LOCALDIR $PROXYURL
   rc=$?
fi

if [[ $protocol == "http" || $protocol == "https" ]]; then
   if [ ! -z $port ]; then
      $cmd_ECHO > /dev/tcp/${host}/${port}
      if [ $? -ne 0 ]; then
         addlog "Wrong port: $port, no connectivity."
         exit 1
      fi
      host="${host}:${port}"
   else
      $cmd_ECHO > /dev/tcp/${host}/80
      if [ $? -ne 0 ]; then
         addlog "No connectivity to default http port 80."
         exit 1
      fi
   fi
   fn_http_push $ISPULL $protocol $host $user_name $password $remote_path $LOCALDIR $PROXYURL
   rc=$?
fi

if [[ $protocol == "sftp" ]]; then
   fn_sftp_push $ISPULL $host $user_name $password $remote_path $LOCALDIR $file_name $port
   rc=$?
fi

if [[ $protocol == "scp" ]]; then
   fn_scp_push $ISPULL $host $user_name $password $remote_path $LOCALDIR $file_name $port
   rc=$?
fi

if [ $rc -ne 0 ]; then
   exit $rc
fi
addlog "Successful "
exit 0
